@@ -20,6 +20,7 @@
 #include "types.h"
 
 #include "vstring.h"
+#include "optscript.h"
 
 /*
 *   DATA DECLARATIONS
@@ -32,32 +33,41 @@ typedef enum eFieldType { /* extension field content control */
 	FIELD_NAME,
 	FIELD_INPUT_FILE,
 	FIELD_PATTERN,
-	FIELD_COMPACT_INPUT_LINE,
+
+	FIELD_ECTAGS_START,
+	FIELD_COMPACT_INPUT_LINE = FIELD_ECTAGS_START,
 
 	/* EXTENSION FIELDS */
-	FIELD_EXTENSION_START,
-	FIELD_ACCESS = FIELD_EXTENSION_START,
-	FIELD_FILE_SCOPE,
-	FIELD_INHERITANCE,
+	FIELD_JSON_LOOP_START,
+	FIELD_FILE_SCOPE = FIELD_JSON_LOOP_START,
 	FIELD_KIND_LONG,
 	FIELD_KIND,
 	FIELD_LANGUAGE,
-	FIELD_IMPLEMENTATION,
 	FIELD_LINE_NUMBER,
-	FIELD_SIGNATURE,
 	FIELD_SCOPE,
 	FIELD_TYPE_REF,
 	FIELD_KIND_KEY,
+	FIELD_ECTAGS_LOOP_START,
+	FIELD_INHERITANCE = FIELD_ECTAGS_LOOP_START,
+	FIELD_ACCESS,
+	FIELD_IMPLEMENTATION,
+	FIELD_SIGNATURE,
+	FIELD_ECTAGS_LOOP_LAST = FIELD_SIGNATURE,
 
-	/* EXTENSION FIELDS NEWLY INTRODUCED IN UCTAGS */
-	FIELD_ROLES,
-	FIELD_REF_MARK,
+	/* Extension fields newly introduced in Universal Ctags. */
+	FIELDS_UCTAGS_START,
+	FIELD_REF_MARK = FIELDS_UCTAGS_START,
 	FIELD_SCOPE_KEY,
+	FIELD_SCOPE_KIND_LONG,
+	FIELD_UCTAGS_LOOP_START,
+	FIELD_ROLES = FIELD_UCTAGS_LOOP_START,
 	FIELD_EXTRAS,
 	FIELD_XPATH,
-	FIELD_SCOPE_KIND_LONG,
 	FIELD_END_LINE,
-	FIELD_BUILTIN_LAST = FIELD_END_LINE,
+	FIELD_EPOCH,
+	FIELD_NTH,
+
+	FIELD_BUILTIN_LAST = FIELD_NTH,
 } fieldType ;
 
 #define fieldDataTypeFalgs "sib" /* used in --list-fields */
@@ -90,6 +100,15 @@ struct sFieldDefinition {
 
 	bool (* isValueAvailable) (const tagEntryInfo *const);
 
+	const char * getterValueType;
+	EsObject * (* getValueObject) (const tagEntryInfo *, const fieldDefinition *);
+	const char * setterValueType;
+
+	/* Return es_false if passed value is acceptable.
+	   Return an error object is unacceptable. */
+	EsObject * (* checkValueForSetter) (const fieldDefinition *, const EsObject *);
+	EsObject * (* setValueObject) (tagEntryInfo *, const fieldDefinition *, const EsObject *);
+
 	fieldDataType dataType; /* used in json output */
 
 	unsigned int ftype;	/* Given from the main part */
