@@ -43,8 +43,10 @@ struct sHashTable {
 	unsigned int size;
 	hashTableHashFunc hashfn;
 	hashTableEqualFunc equalfn;
-	hashTableFreeFunc keyfreefn;
-	hashTableFreeFunc valfreefn;
+	hashTableDeleteFunc keyfreefn;
+	hashTableDeleteFunc valfreefn;
+	void *valForNotUnknownKey;
+	hashTableDeleteFunc valForNotUnknownKeyfreefn;
 };
 
 struct chainTracker {
@@ -65,18 +67,27 @@ static hentry* entry_new (void *key, void *value, hentry* next)
 	return entry;
 }
 
-static hentry* entry_destroy (hentry* entry,
-			      hashTableFreeFunc keyfreefn,
-			      hashTableFreeFunc valfreefn)
+static void entry_reset  (hentry* entry,
+						  void *newkey,
+						  void *newval,
+						  hashTableDeleteFunc keyfreefn,
+						  hashTableDeleteFunc valfreefn)
 {
-	hentry* tmp;
-
 	if (keyfreefn)
 		keyfreefn (entry->key);
 	if (valfreefn)
 		valfreefn (entry->value);
-	entry->key = NULL;
-	entry->value = NULL;
+	entry->key = newkey;
+	entry->value = newval;
+}
+
+static hentry* entry_destroy (hentry* entry,
+			      hashTableDeleteFunc keyfreefn,
+			      hashTableDeleteFunc valfreefn)
+{
+	hentry* tmp;
+
+	entry_reset (entry, NULL, NULL, keyfreefn, valfreefn);
 	tmp = entry->next;
 	eFree (entry);
 
@@ -84,14 +95,15 @@ static hentry* entry_destroy (hentry* entry,
 }
 
 static void  entry_reclaim (hentry* entry,
-			    hashTableFreeFunc keyfreefn,
-			    hashTableFreeFunc valfreefn)
+			    hashTableDeleteFunc keyfreefn,
+			    hashTableDeleteFunc valfreefn)
 {
 	while (entry)
 		entry = entry_destroy (entry, keyfreefn, valfreefn);
 }
 
-static void *entry_find (hentry* entry, const void* const key, hashTableEqualFunc equalfn)
+static void *entry_find (hentry* entry, const void* const key, hashTableEqualFunc equalfn,
+						 void *valForNotUnknownKey)
 {
 	while (entry)
 	{
@@ -99,11 +111,11 @@ static void *entry_find (hentry* entry, const void* const key, hashTableEqualFun
 			return entry->value;
 		entry = entry->next;
 	}
-	return NULL;
+	return valForNotUnknownKey;
 }
 
 static bool		entry_delete (hentry **entry, const void *key, hashTableEqualFunc equalfn,
-			      hashTableFreeFunc keyfreefn, hashTableFreeFunc valfreefn)
+			      hashTableDeleteFunc keyfreefn, hashTableDeleteFunc valfreefn)
 {
 	while (*entry)
 	{
@@ -112,7 +124,22 @@ static bool		entry_delete (hentry **entry, const void *key, hashTableEqualFunc e
 			*entry = entry_destroy (*entry, keyfreefn, valfreefn);
 			return true;
 		}
+		entry = &((*entry)->next);
+	}
+	return false;
+}
 
+static bool		entry_update (hentry *entry, void *key, void *value, hashTableEqualFunc equalfn,
+			      hashTableDeleteFunc keyfreefn, hashTableDeleteFunc valfreefn)
+{
+	while (entry)
+	{
+		if (equalfn (key, entry->key))
+		{
+			entry_reset (entry, key, value, keyfreefn, valfreefn);
+			return true;
+		}
+		entry = entry->next;
 	}
 	return false;
 }
@@ -131,8 +158,8 @@ static bool  entry_foreach (hentry *entry, hashTableForeachFunc proc, void *user
 extern hashTable *hashTableNew    (unsigned int size,
 				   hashTableHashFunc hashfn,
 				   hashTableEqualFunc equalfn,
-				   hashTableFreeFunc keyfreefn,
-				   hashTableFreeFunc valfreefn)
+				   hashTableDeleteFunc keyfreefn,
+				   hashTableDeleteFunc valfreefn)
 {
 	hashTable *htable;
 
@@ -144,16 +171,21 @@ extern hashTable *hashTableNew    (unsigned int size,
 	htable->equalfn = equalfn;
 	htable->keyfreefn = keyfreefn;
 	htable->valfreefn = valfreefn;
+	htable->valForNotUnknownKey = NULL;
+	htable->valForNotUnknownKeyfreefn = NULL;
 
 	return htable;
 }
 
-extern hashTable* hashTableIntNew (unsigned int size,
-								   hashTableHashFunc hashfn,
-								   hashTableEqualFunc equalfn,
-								   hashTableFreeFunc keyfreefn)
+extern void hashTableSetValueForUnknownKey (hashTable *htable,
+											void *val,
+											hashTableDeleteFunc valfreefn)
 {
-	return hashTableNew (size, hashfn, equalfn, keyfreefn, NULL);
+	if (htable->valfreefn)
+		htable->valfreefn (htable->valForNotUnknownKey);
+
+	htable->valForNotUnknownKey = val;
+	htable->valForNotUnknownKeyfreefn = valfreefn;
 }
 
 extern void       hashTableDelete (hashTable *htable)
@@ -163,6 +195,8 @@ extern void       hashTableDelete (hashTable *htable)
 
 	hashTableClear (htable);
 
+	if (htable->valForNotUnknownKeyfreefn)
+		htable->valForNotUnknownKeyfreefn (htable->valForNotUnknownKey);
 	eFree (htable->table);
 	eFree (htable);
 }
@@ -196,7 +230,7 @@ extern void*      hashTableGetItem   (hashTable *htable, const void * key)
 	unsigned int i;
 
 	i = htable->hashfn (key) % htable->size;
-	return entry_find(htable->table[i], key, htable->equalfn);
+	return entry_find(htable->table[i], key, htable->equalfn, htable->valForNotUnknownKey);
 }
 
 extern bool     hashTableDeleteItem (hashTable *htable, const void *key)
@@ -208,9 +242,21 @@ extern bool     hashTableDeleteItem (hashTable *htable, const void *key)
 			    htable->equalfn, htable->keyfreefn, htable->valfreefn);
 }
 
+extern bool    hashTableUpdateItem (hashTable *htable, void *key, void *value)
+{
+	unsigned int i;
+
+	i = htable->hashfn (key) % htable->size;
+	bool r = entry_update(htable->table[i], key, value,
+						  htable->equalfn, htable->keyfreefn, htable->valfreefn);
+	if (!r)
+		htable->table[i] = entry_new(key, value, htable->table[i]);
+	return r;
+}
+
 extern bool    hashTableHasItem    (hashTable *htable, const void *key)
 {
-	return hashTableGetItem (htable, key)? true: false;
+	return hashTableGetItem (htable, key) == htable->valForNotUnknownKey? false: true;
 }
 
 extern bool       hashTableForeachItem (hashTable *htable, hashTableForeachFunc proc, void *user_data)
@@ -258,7 +304,7 @@ static bool count (const void *const key CTAGS_ATTR_UNUSED, void *value CTAGS_AT
 	return true;
 }
 
-extern int        hashTableCountItem   (hashTable *htable)
+extern unsigned int hashTableCountItem   (hashTable *htable)
 {
 	int c = 0;
 	hashTableForeachItem (htable, count, &c);
