@@ -21,6 +21,7 @@
 #include "xtag.h"
 #include "numarray.h"
 #include "routines.h"
+#include "options.h"
 
 #include <string.h>
 
@@ -68,7 +69,11 @@ int  makePromise   (const char *parser,
 {
 	struct promise *p;
 	int r;
-	langType lang;
+	langType lang = LANG_IGNORE;
+
+	verbose("makePromise: %s start(line: %lu, offset: %lu, srcline: %lu), end(line: %lu, offset: %lu)\n",
+			parser? parser: "*", startLine, startCharOffset, sourceLineOffset,
+			endLine, endCharOffset);
 
 	if ((!isThinStreamSpec(startLine,
 						   startCharOffset,
@@ -78,9 +83,12 @@ int  makePromise   (const char *parser,
 		&& ( !isXtagEnabled (XTAG_GUEST)))
 		return -1;
 
-	lang = getNamedLanguage (parser, 0);
-	if (lang == LANG_IGNORE)
-		return -1;
+	if (parser)
+	{
+		lang = getNamedLanguage (parser, 0);
+		if (lang == LANG_IGNORE)
+			return -1;
+	}
 
 	if ( promise_count == promise_allocated)
 	{
@@ -164,15 +172,17 @@ bool forcePromises (void)
 	{
 		current_promise = i;
 		struct promise *p = promises + i;
-		tagFileResized = runParserInNarrowedInputStream (p->lang,
-								 p->startLine,
-								 p->startCharOffset,
-								 p->endLine,
-								 p->endCharOffset,
-								 p->sourceLineOffset,
-								 i)
-			? true
-			: tagFileResized;
+
+		if (p->lang != LANG_IGNORE && isLanguageEnabled (p->lang))
+			tagFileResized = runParserInNarrowedInputStream (p->lang,
+															 p->startLine,
+															 p->startCharOffset,
+															 p->endLine,
+															 p->endCharOffset,
+															 p->sourceLineOffset,
+															 i)
+				? true
+				: tagFileResized;
 	}
 
 	freeModifiers (0);
@@ -306,3 +316,12 @@ void runModifiers (int promise,
 	}
 	ptrArrayDelete (modifiers);
 }
+
+void promiseUpdateLanguage  (int promise, langType lang)
+{
+	Assert (promise >= 0);
+
+	struct promise *p = promises + promise;
+
+	p->lang = lang;
+}
