@@ -112,6 +112,7 @@ typedef struct sInputFile {
 	inputLineFposMap lineFposMap;
 	vString *allLines;
 	int thinDepth;
+	time_t mtime;
 } inputFile;
 
 static inputLangInfo inputLang;
@@ -190,7 +191,7 @@ extern MIOPos getInputFilePositionForLine (unsigned int line)
 extern long getInputFileOffsetForLine (unsigned int line)
 {
 	compoundPos *cpos = getInputFileCompoundPosForLine (line);
-	return cpos->offset;
+	return cpos->offset - (File.bomFound? 3: 0);
 }
 
 extern langType getInputLanguage (void)
@@ -361,6 +362,9 @@ extern unsigned long getInputLineNumberForFileOffset(long offset)
 {
 	compoundPos *p;
 
+	if (File.bomFound)
+		offset += 3;
+
 	p = bsearch (&offset, File.lineFposMap.pos, File.lineFposMap.count, sizeof (compoundPos),
 		     compoundPosForOffset);
 	if (p == NULL)
@@ -604,8 +608,8 @@ static bool parseLineDirective (char *s)
 #define MAX_IN_MEMORY_FILE_SIZE (1024*1024)
 #endif
 
-extern MIO *getMio (const char *const fileName, const char *const openMode,
-		    bool memStreamRequired)
+static MIO *getMioFull (const char *const fileName, const char *const openMode,
+		    bool memStreamRequired, time_t *mtime)
 {
 	FILE *src;
 	fileStatus *st;
@@ -614,6 +618,8 @@ extern MIO *getMio (const char *const fileName, const char *const openMode,
 
 	st = eStat (fileName);
 	size = st->size;
+	if (mtime)
+		*mtime = st->mtime;
 	eStatFree (st);
 	if ((!memStreamRequired)
 	    && (size > MAX_IN_MEMORY_FILE_SIZE || size == 0))
@@ -637,6 +643,12 @@ extern MIO *getMio (const char *const fileName, const char *const openMode,
 	return mio_new_memory (data, size, eRealloc, eFreeNoNullCheck);
 }
 
+extern MIO *getMio (const char *const fileName, const char *const openMode,
+		    bool memStreamRequired)
+{
+	return getMioFull (fileName, openMode, memStreamRequired, NULL);
+}
+
 /* Return true if utf8 BOM is found */
 static bool checkUTF8BOM (MIO *mio, bool skipIfFound)
 {
@@ -671,7 +683,7 @@ static void rewindInputFile (inputFile *f)
  *  fails, it will display an error message and leave the File.mio set to NULL.
  */
 extern bool openInputFile (const char *const fileName, const langType language,
-			      MIO *mio)
+			      MIO *mio, time_t mtime)
 {
 	const char *const openMode = "rb";
 	bool opened = false;
@@ -708,7 +720,7 @@ extern bool openInputFile (const char *const fileName, const langType language,
 			mio_rewind (mio);
 	}
 
-	File.mio = mio? mio_ref (mio): getMio (fileName, openMode, memStreamRequired);
+	File.mio = mio? mio_ref (mio): getMioFull (fileName, openMode, memStreamRequired, &File.mtime);
 
 	if (File.mio == NULL)
 		error (WARNING | PERROR, "cannot open \"%s\"", fileName);
@@ -716,6 +728,8 @@ extern bool openInputFile (const char *const fileName, const langType language,
 	{
 		opened = true;
 
+		if (File.mio == mio)
+			File.mtime = mtime;
 
 		File.bomFound = checkUTF8BOM (File.mio, true);
 
@@ -748,6 +762,11 @@ extern bool openInputFile (const char *const fileName, const langType language,
 	return opened;
 }
 
+extern time_t getInputFileMtime (void)
+{
+	return File.mtime;
+}
+
 extern void resetInputFile (const langType language)
 {
 	Assert (File.mio);
@@ -875,7 +894,7 @@ static eolType readLine (vString *const vLine, MIO *const mio)
 	return r;
 }
 
-static vString *iFileGetLine (void)
+static vString *iFileGetLine (bool chop_newline)
 {
 	eolType eol;
 	langType lang = getInputLanguage();
@@ -895,11 +914,17 @@ static vString *iFileGetLine (void)
 
 		if (Option.lineDirectives && vStringChar (File.line, 0) == '#')
 			parseLineDirective (vStringValue (File.line) + 1);
-		matchLanguageRegex (lang, File.line);
 
 		if (File.allLines)
 			vStringCat (File.allLines, File.line);
 
+		bool chopped = vStringStripNewline (File.line);
+
+		matchLanguageRegex (lang, File.line);
+
+		if (chopped && !chop_newline)
+			vStringPutNewlinAgainUnsafe (File.line);
+
 		return File.line;
 	}
 	else
@@ -943,7 +968,7 @@ extern int getcFromInputFile (void)
 		}
 		else
 		{
-			vString* const line = iFileGetLine ();
+			vString* const line = iFileGetLine (false);
 			if (line != NULL)
 				File.currentLine = (unsigned char*) vStringValue (line);
 			if (File.currentLine == NULL)
@@ -999,12 +1024,11 @@ extern int skipToCharacterInInputFile2 (int c0, int c1)
  */
 extern const unsigned char *readLineFromInputFile (void)
 {
-	vString* const line = iFileGetLine ();
+	vString* const line = iFileGetLine (true);
 	const unsigned char* result = NULL;
 	if (line != NULL)
 	{
 		result = (const unsigned char*) vStringValue (line);
-		vStringStripNewline (line);
 		DebugStatement ( debugPrintf (DEBUG_READ, "%s\n", result); )
 	}
 	return result;
