@@ -10,7 +10,6 @@
 *   INCLUDE FILES
 */
 #include "general.h"	/* must always come first */
-#include "main.h"
 
 #include <string.h>
 
@@ -59,7 +58,7 @@ static kindDefinition rustKinds[] = {
 	{true, 'M', "macro", "Macro Definition"},
 	{true, 'm', "field", "A struct field"},
 	{true, 'e', "enumerator", "An enum variant"},
-	{true, 'F', "method", "A method"},
+	{true, 'P', "method", "A method"},
 };
 
 typedef enum {
@@ -97,8 +96,7 @@ static void parseBlock (lexerState *lexer, bool delim, int kind, vString *scope)
 /* Resets the scope string to the old length */
 static void resetScope (vString *scope, size_t old_len)
 {
-	scope->length = old_len;
-	scope->buffer[old_len] = '\0';
+	vStringTruncate (scope, old_len);
 }
 
 /* Adds a name to the end of the scope string */
@@ -434,23 +432,22 @@ static void deInitLexer (lexerState *lexer)
 	lexer->token_str = NULL;
 }
 
-static void addTag (vString* ident, const char* type, const char* arg_list, int kind, unsigned long line, MIOPos pos, vString *scope, int parent_kind)
+static void addTag (vString* ident, const char* arg_list, int kind, unsigned long line, MIOPos pos, vString *scope, int parent_kind)
 {
-	if (kind == K_NONE)
+	if (kind == K_NONE || ! rustKinds[kind].enabled)
 		return;
 	tagEntryInfo tag;
-	initTagEntry(&tag, ident->buffer, kind);
+	initTagEntry(&tag, vStringValue(ident), kind);
 
 	tag.lineNumber = line;
 	tag.filePosition = pos;
-	tag.sourceFileName = getInputFileName();
 
 	tag.extensionFields.signature = arg_list;
-	tag.extensionFields.varType = type;
+	/*tag.extensionFields.varType = type;*/ /* FIXME: map to typeRef[1]? */
 	if (parent_kind != K_NONE)
 	{
 		tag.extensionFields.scopeKindIndex = parent_kind;
-		tag.extensionFields.scopeName = scope->buffer;
+		tag.extensionFields.scopeName = vStringValue(scope);
 	}
 	makeTagEntry(&tag);
 }
@@ -531,6 +528,7 @@ static void parseFn (lexerState *lexer, vString *scope, int parent_kind)
 	unsigned long line;
 	MIOPos pos;
 	int paren_level = 0;
+	int bracket_level = 0;
 	bool found_paren = false;
 	bool valid_signature = true;
 
@@ -548,9 +546,13 @@ static void parseFn (lexerState *lexer, vString *scope, int parent_kind)
 
 	/* HACK: This is a bit coarse as far as what tag entry means by
 	 * 'arglist'... */
-	while (lexer->cur_token != '{' && lexer->cur_token != ';')
+	while (lexer->cur_token != '{')
 	{
-		if (lexer->cur_token == '}')
+		if (lexer->cur_token == ';' && bracket_level == 0)
+		{
+			break;
+		}
+		else if (lexer->cur_token == '}')
 		{
 			valid_signature = false;
 			break;
@@ -569,6 +571,14 @@ static void parseFn (lexerState *lexer, vString *scope, int parent_kind)
 				break;
 			}
 		}
+		else if (lexer->cur_token == '[')
+		{
+			bracket_level++;
+		}
+		else if (lexer->cur_token == ']')
+		{
+			bracket_level--;
+		}
 		else if (lexer->cur_token == TOKEN_EOF)
 		{
 			valid_signature = false;
@@ -577,13 +587,13 @@ static void parseFn (lexerState *lexer, vString *scope, int parent_kind)
 		writeCurTokenToStr(lexer, arg_list);
 		advanceToken(lexer, false);
 	}
-	if (!found_paren || paren_level != 0)
+	if (!found_paren || paren_level != 0 || bracket_level != 0)
 		valid_signature = false;
 
 	if (valid_signature)
 	{
 		vStringStripTrailing(arg_list);
-		addTag(name, NULL, arg_list->buffer, kind, line, pos, scope, parent_kind);
+		addTag(name, vStringValue(arg_list), kind, line, pos, scope, parent_kind);
 		addToScope(scope, name);
 		parseBlock(lexer, true, kind, scope);
 	}
@@ -601,7 +611,7 @@ static void parseMod (lexerState *lexer, vString *scope, int parent_kind)
 	if (lexer->cur_token != TOKEN_IDENT)
 		return;
 
-	addTag(lexer->token_str, NULL, NULL, K_MOD, lexer->line, lexer->pos, scope, parent_kind);
+	addTag(lexer->token_str, NULL, K_MOD, lexer->line, lexer->pos, scope, parent_kind);
 	addToScope(scope, lexer->token_str);
 
 	advanceToken(lexer, true);
@@ -620,7 +630,7 @@ static void parseTrait (lexerState *lexer, vString *scope, int parent_kind)
 	if (lexer->cur_token != TOKEN_IDENT)
 		return;
 
-	addTag(lexer->token_str, NULL, NULL, K_TRAIT, lexer->line, lexer->pos, scope, parent_kind);
+	addTag(lexer->token_str, NULL, K_TRAIT, lexer->line, lexer->pos, scope, parent_kind);
 	addToScope(scope, lexer->token_str);
 
 	advanceToken(lexer, true);
@@ -648,8 +658,8 @@ static void parseQualifiedType (lexerState *lexer, vString* name)
 	{
 		if (lexer->cur_token == TOKEN_IDENT)
 		{
-			if (strcmp(lexer->token_str->buffer, "for") == 0
-				|| strcmp(lexer->token_str->buffer, "where") == 0)
+			if (strcmp(vStringValue(lexer->token_str), "for") == 0
+				|| strcmp(vStringValue(lexer->token_str), "where") == 0)
 				break;
 			vStringClear(name);
 			vStringCat(name, lexer->token_str);
@@ -683,13 +693,13 @@ static void parseImpl (lexerState *lexer, vString *scope, int parent_kind)
 
 	parseQualifiedType(lexer, name);
 
-	if (lexer->cur_token == TOKEN_IDENT && strcmp(lexer->token_str->buffer, "for") == 0)
+	if (lexer->cur_token == TOKEN_IDENT && strcmp(vStringValue(lexer->token_str), "for") == 0)
 	{
 		advanceToken(lexer, true);
 		parseQualifiedType(lexer, name);
 	}
 
-	addTag(name, NULL, NULL, K_IMPL, line, pos, scope, parent_kind);
+	addTag(name, NULL, K_IMPL, line, pos, scope, parent_kind);
 	addToScope(scope, name);
 
 	parseBlock(lexer, true, K_IMPL, scope);
@@ -705,14 +715,14 @@ static void parseStatic (lexerState *lexer, vString *scope, int parent_kind)
 	advanceToken(lexer, true);
 	if (lexer->cur_token != TOKEN_IDENT)
 		return;
-	if (strcmp(lexer->token_str->buffer, "mut") == 0)
+	if (strcmp(vStringValue(lexer->token_str), "mut") == 0)
 	{
 		advanceToken(lexer, true);
 	}
 	if (lexer->cur_token != TOKEN_IDENT)
 		return;
 
-	addTag(lexer->token_str, NULL, NULL, K_STATIC, lexer->line, lexer->pos, scope, parent_kind);
+	addTag(lexer->token_str, NULL, K_STATIC, lexer->line, lexer->pos, scope, parent_kind);
 }
 
 /* Type format:
@@ -724,7 +734,7 @@ static void parseType (lexerState *lexer, vString *scope, int parent_kind)
 	if (lexer->cur_token != TOKEN_IDENT)
 		return;
 
-	addTag(lexer->token_str, NULL, NULL, K_TYPE, lexer->line, lexer->pos, scope, parent_kind);
+	addTag(lexer->token_str, NULL, K_TYPE, lexer->line, lexer->pos, scope, parent_kind);
 }
 
 /* Structs and enums are very similar syntax-wise.
@@ -746,7 +756,7 @@ static void parseStructOrEnum (lexerState *lexer, vString *scope, int parent_kin
 	if (lexer->cur_token != TOKEN_IDENT)
 		return;
 
-	addTag(lexer->token_str, NULL, NULL, kind, lexer->line, lexer->pos, scope, parent_kind);
+	addTag(lexer->token_str, NULL, kind, lexer->line, lexer->pos, scope, parent_kind);
 	addToScope(scope, lexer->token_str);
 
 	skipUntil(lexer, goal_tokens1, 2);
@@ -779,10 +789,20 @@ static void parseStructOrEnum (lexerState *lexer, vString *scope, int parent_kin
 			}
 			if (lexer->cur_token == TOKEN_IDENT)
 			{
-				if (strcmp(lexer->token_str->buffer, "priv") == 0
-				    || strcmp(lexer->token_str->buffer, "pub") == 0)
+				if (strcmp(vStringValue(lexer->token_str), "priv") == 0
+				    || strcmp(vStringValue(lexer->token_str), "pub") == 0)
 				{
 					advanceToken(lexer, true);
+
+					/* Skip thevisibility specificaions.
+					 * https://doc.rust-lang.org/reference/visibility-and-privacy.html */
+					if (lexer->cur_token == '(')
+					{
+						advanceToken(lexer, true);
+						skipUntil (lexer, (int []){')'}, 1);
+						advanceToken(lexer, true);
+					}
+
 					if (lexer->cur_token != TOKEN_IDENT)
 					{
 						/* Something's up with this field, skip to the next one */
@@ -793,7 +813,7 @@ static void parseStructOrEnum (lexerState *lexer, vString *scope, int parent_kin
 
 				vStringClear(field_name);
 				vStringCat(field_name, lexer->token_str);
-				addTag(field_name, NULL, NULL, field_kind, lexer->line, lexer->pos, scope, kind);
+				addTag(field_name, NULL, field_kind, lexer->line, lexer->pos, scope, kind);
 				skipUntil(lexer, goal_tokens2, 2);
 			}
 			if (lexer->cur_token == '}')
@@ -864,7 +884,7 @@ static void parseMacroRules (lexerState *lexer, vString *scope, int parent_kind)
 	if (lexer->cur_token != TOKEN_IDENT)
 		return;
 
-	addTag(lexer->token_str, NULL, NULL, K_MACRO, lexer->line, lexer->pos, scope, parent_kind);
+	addTag(lexer->token_str, NULL, K_MACRO, lexer->line, lexer->pos, scope, parent_kind);
 
 	skipMacro(lexer);
 }
@@ -886,39 +906,39 @@ static void parseBlock (lexerState *lexer, bool delim, int kind, vString *scope)
 		if (lexer->cur_token == TOKEN_IDENT)
 		{
 			size_t old_scope_len = vStringLength(scope);
-			if (strcmp(lexer->token_str->buffer, "fn") == 0)
+			if (strcmp(vStringValue(lexer->token_str), "fn") == 0)
 			{
 				parseFn(lexer, scope, kind);
 			}
-			else if(strcmp(lexer->token_str->buffer, "mod") == 0)
+			else if(strcmp(vStringValue(lexer->token_str), "mod") == 0)
 			{
 				parseMod(lexer, scope, kind);
 			}
-			else if(strcmp(lexer->token_str->buffer, "static") == 0)
+			else if(strcmp(vStringValue(lexer->token_str), "static") == 0)
 			{
 				parseStatic(lexer, scope, kind);
 			}
-			else if(strcmp(lexer->token_str->buffer, "trait") == 0)
+			else if(strcmp(vStringValue(lexer->token_str), "trait") == 0)
 			{
 				parseTrait(lexer, scope, kind);
 			}
-			else if(strcmp(lexer->token_str->buffer, "type") == 0)
+			else if(strcmp(vStringValue(lexer->token_str), "type") == 0)
 			{
 				parseType(lexer, scope, kind);
 			}
-			else if(strcmp(lexer->token_str->buffer, "impl") == 0)
+			else if(strcmp(vStringValue(lexer->token_str), "impl") == 0)
 			{
 				parseImpl(lexer, scope, kind);
 			}
-			else if(strcmp(lexer->token_str->buffer, "struct") == 0)
+			else if(strcmp(vStringValue(lexer->token_str), "struct") == 0)
 			{
 				parseStructOrEnum(lexer, scope, kind, true);
 			}
-			else if(strcmp(lexer->token_str->buffer, "enum") == 0)
+			else if(strcmp(vStringValue(lexer->token_str), "enum") == 0)
 			{
 				parseStructOrEnum(lexer, scope, kind, false);
 			}
-			else if(strcmp(lexer->token_str->buffer, "macro_rules") == 0)
+			else if(strcmp(vStringValue(lexer->token_str), "macro_rules") == 0)
 			{
 				parseMacroRules(lexer, scope, kind);
 			}
@@ -946,7 +966,7 @@ static void parseBlock (lexerState *lexer, bool delim, int kind, vString *scope)
 		{
 			/* Skip over the 'static lifetime, as it confuses the static parser above */
 			advanceToken(lexer, true);
-			if (lexer->cur_token == TOKEN_IDENT && strcmp(lexer->token_str->buffer, "static") == 0)
+			if (lexer->cur_token == TOKEN_IDENT && strcmp(vStringValue(lexer->token_str), "static") == 0)
 				advanceToken(lexer, true);
 		}
 		else
@@ -960,7 +980,7 @@ static void parseBlock (lexerState *lexer, bool delim, int kind, vString *scope)
 
 static void findRustTags (void)
 {
-	lexerState lexer;
+	lexerState lexer = {0};
 	vString* scope = vStringNew();
 	initLexer(&lexer);
 
@@ -973,7 +993,7 @@ static void findRustTags (void)
 extern parserDefinition *RustParser (void)
 {
 	static const char *const extensions[] = { "rs", NULL };
-	parserDefinition *def = parserNewFull ("Rust", KIND_FILE_ALT);
+	parserDefinition *def = parserNew ("Rust");
 	def->kindTable = rustKinds;
 	def->kindCount = ARRAY_SIZE (rustKinds);
 	def->extensions = extensions;
