@@ -30,6 +30,9 @@
  * - it will be loaded at next startup.
  */
 
+#ifdef HAVE_CONFIG_H
+# include "config.h"
+#endif
 
 #include "geanyplugin.h"	/* plugin API, always comes first */
 #include "Scintilla.h"	/* for the SCNotification struct */
@@ -49,7 +52,7 @@ static gboolean on_editor_notify(GObject *object, GeanyEditor *editor,
 	GeanyData *geany_data = plugin->geany_data;
 
 	/* For detailed documentation about the SCNotification struct, please see
-	 * http://www.scintilla.org/ScintillaDoc.html#Notifications. */
+	 * https://www.scintilla.org/ScintillaDoc.html#Notifications. */
 	switch (nt->nmhdr.code)
 	{
 		case SCN_UPDATEUI:
@@ -182,7 +185,7 @@ static GtkWidget *demo_configure(GeanyPlugin *plugin, GtkDialog *dialog, gpointe
 	GtkWidget *label, *entry, *vbox;
 
 	/* example configuration dialog */
-	vbox = gtk_vbox_new(FALSE, 6);
+	vbox = gtk_box_new(GTK_ORIENTATION_VERTICAL, 6);
 
 	/* add a label and a text entry to the dialog */
 	label = gtk_label_new(_("Welcome text to show:"));
@@ -216,7 +219,7 @@ static void demo_cleanup(GeanyPlugin *plugin, gpointer data)
 void geany_load_module(GeanyPlugin *plugin)
 {
 	/* main_locale_init() must be called for your package before any localization can be done */
-	main_locale_init(LOCALEDIR, GETTEXT_PACKAGE);
+	main_locale_init(GEANY_LOCALEDIR, GETTEXT_PACKAGE);
 	plugin->info->name = _("Demo");
 	plugin->info->description = _("Example plugin.");
 	plugin->info->version = "0.4";
