@@ -7,19 +7,27 @@
 
 #include <cstdlib>
 #include <cstring>
+#include <climits>
 
 #include <stdexcept>
+#include <string_view>
 #include <vector>
 #include <map>
+#include <optional>
 #include <algorithm>
 #include <iterator>
 #include <memory>
 
+#include "ScintillaTypes.h"
+
+#include "Debugging.h"
+#include "Geometry.h"
 #include "Platform.h"
 
 #include "XPM.h"
 
 using namespace Scintilla;
+using namespace Scintilla::Internal;
 
 namespace {
 
@@ -56,17 +64,17 @@ unsigned int ValueOfHex(const char ch) noexcept {
 		return 0;
 }
 
-ColourDesired ColourFromHex(const char *val) noexcept {
+ColourRGBA ColourFromHex(const char *val) noexcept {
 	const unsigned int r = ValueOfHex(val[0]) * 16 + ValueOfHex(val[1]);
 	const unsigned int g = ValueOfHex(val[2]) * 16 + ValueOfHex(val[3]);
 	const unsigned int b = ValueOfHex(val[4]) * 16 + ValueOfHex(val[5]);
-	return ColourDesired(r, g, b);
+	return ColourRGBA(r, g, b);
 }
 
 }
 
 
-ColourDesired XPM::ColourFromCode(int ch) const noexcept {
+ColourRGBA XPM::ColourFromCode(int ch) const noexcept {
 	return colourCodeTable[ch];
 }
 
@@ -85,9 +93,6 @@ XPM::XPM(const char *const *linesForm) {
 	Init(linesForm);
 }
 
-XPM::~XPM() {
-}
-
 void XPM::Init(const char *textForm) {
 	// Test done is two parts to avoid possibility of overstepping the memory
 	// if memcmp implemented strangely. Must be 4 bytes at least at destination.
@@ -95,7 +100,7 @@ void XPM::Init(const char *textForm) {
 		// Build the lines form out of the text form
 		std::vector<const char *> linesForm = LinesFormFromTextForm(textForm);
 		if (!linesForm.empty()) {
-			Init(&linesForm[0]);
+			Init(linesForm.data());
 		}
 	} else {
 		// It is really in line form
@@ -112,7 +117,7 @@ void XPM::Init(const char *const *linesForm) {
 	if (!linesForm)
 		return;
 
-	std::fill(colourCodeTable, std::end(colourCodeTable), ColourDesired(0));
+	std::fill(colourCodeTable, std::end(colourCodeTable), ColourRGBA(0, 0, 0));
 	const char *line0 = linesForm[0];
 	width = atoi(line0);
 	line0 = NextField(line0);
@@ -130,7 +135,7 @@ void XPM::Init(const char *const *linesForm) {
 		const char *colourDef = linesForm[c+1];
 		const char code = colourDef[0];
 		colourDef += 4;
-		ColourDesired colour(0xff, 0xff, 0xff);
+		ColourRGBA colour(0, 0, 0, 0);
 		if (*colourDef == '#') {
 			colour = ColourFromHex(colourDef+1);
 		} else {
@@ -139,7 +144,7 @@ void XPM::Init(const char *const *linesForm) {
 		colourCodeTable[static_cast<unsigned char>(code)] = colour;
 	}
 
-	for (int y=0; y<height; y++) {
+	for (ptrdiff_t y=0; y<height; y++) {
 		const char *lform = linesForm[y+nColours+1];
 		const size_t len = MeasureLength(lform);
 		for (size_t x = 0; x<len; x++)
@@ -169,19 +174,13 @@ void XPM::Draw(Surface *surface, const PRectangle &rc) {
 	}
 }
 
-void XPM::PixelAt(int x, int y, ColourDesired &colour, bool &transparent) const noexcept {
-	if (pixels.empty() || (x<0) || (x >= width) || (y<0) || (y >= height)) {
-		colour = ColourDesired(0);
-		transparent = true;
-		return;
+ColourRGBA XPM::PixelAt(int x, int y) const noexcept {
+	if (pixels.empty() || (x < 0) || (x >= width) || (y < 0) || (y >= height)) {
+		// Out of bounds -> transparent black
+		return ColourRGBA(0, 0, 0, 0);
 	}
 	const int code = pixels[y * width + x];
-	transparent = code == codeTransparent;
-	if (transparent) {
-		colour = ColourDesired(0);
-	} else {
-		colour = ColourFromCode(code);
-	}
+	return ColourFromCode(code);
 }
 
 std::vector<const char *> XPM::LinesFormFromTextForm(const char *textForm) {
@@ -235,15 +234,17 @@ RGBAImage::RGBAImage(const XPM &xpm) {
 	pixelBytes.resize(CountBytes());
 	for (int y=0; y<height; y++) {
 		for (int x=0; x<width; x++) {
-			ColourDesired colour;
-			bool transparent = false;
-			xpm.PixelAt(x, y, colour, transparent);
-			SetPixel(x, y, colour, transparent ? 0 : 255);
+			SetPixel(x, y, xpm.PixelAt(x, y));
 		}
 	}
 }
 
-RGBAImage::~RGBAImage() {
+float RGBAImage::GetScaledHeight() const noexcept {
+	return static_cast<float>(height) / scale;
+}
+
+float RGBAImage::GetScaledWidth() const noexcept {
+	return static_cast<float>(width) / scale;
 }
 
 int RGBAImage::CountBytes() const noexcept {
@@ -251,16 +252,24 @@ int RGBAImage::CountBytes() const noexcept {
 }
 
 const unsigned char *RGBAImage::Pixels() const noexcept {
-	return &pixelBytes[0];
+	return pixelBytes.data();
 }
 
-void RGBAImage::SetPixel(int x, int y, ColourDesired colour, int alpha) noexcept {
-	unsigned char *pixel = &pixelBytes[0] + (y*width+x) * 4;
+void RGBAImage::SetPixel(int x, int y, ColourRGBA colour) noexcept {
+	unsigned char *pixel = pixelBytes.data() + (y * width + x) * 4;
 	// RGBA
 	pixel[0] = colour.GetRed();
 	pixel[1] = colour.GetGreen();
 	pixel[2] = colour.GetBlue();
-	pixel[3] = static_cast<unsigned char>(alpha);
+	pixel[3] = colour.GetAlpha();
+}
+
+namespace {
+
+constexpr unsigned char AlphaMultiplied(unsigned char value, unsigned char alpha) noexcept {
+	return (value * alpha / UCHAR_MAX) & 0xffU;
+}
+
 }
 
 // Transform a block of pixels from RGBA to BGRA with premultiplied alpha.
@@ -269,9 +278,9 @@ void RGBAImage::BGRAFromRGBA(unsigned char *pixelsBGRA, const unsigned char *pix
 	for (size_t i = 0; i < count; i++) {
 		const unsigned char alpha = pixelsRGBA[3];
 		// Input is RGBA, output is BGRA with premultiplied alpha
-		pixelsBGRA[2] = pixelsRGBA[0] * alpha / 255;
-		pixelsBGRA[1] = pixelsRGBA[1] * alpha / 255;
-		pixelsBGRA[0] = pixelsRGBA[2] * alpha / 255;
+		pixelsBGRA[2] = AlphaMultiplied(pixelsRGBA[0], alpha);
+		pixelsBGRA[1] = AlphaMultiplied(pixelsRGBA[1], alpha);
+		pixelsBGRA[0] = AlphaMultiplied(pixelsRGBA[2], alpha);
 		pixelsBGRA[3] = alpha;
 		pixelsRGBA += bytesPerPixel;
 		pixelsBGRA += bytesPerPixel;
@@ -281,10 +290,6 @@ void RGBAImage::BGRAFromRGBA(unsigned char *pixelsBGRA, const unsigned char *pix
 RGBAImageSet::RGBAImageSet() : height(-1), width(-1) {
 }
 
-RGBAImageSet::~RGBAImageSet() {
-	Clear();
-}
-
 /// Remove all images.
 void RGBAImageSet::Clear() noexcept {
 	images.clear();
@@ -293,13 +298,8 @@ void RGBAImageSet::Clear() noexcept {
 }
 
 /// Add an image.
-void RGBAImageSet::Add(int ident, RGBAImage *image) {
-	ImageMap::iterator it=images.find(ident);
-	if (it == images.end()) {
-		images[ident] = std::unique_ptr<RGBAImage>(image);
-	} else {
-		it->second.reset(image);
-	}
+void RGBAImageSet::AddImage(int ident, std::unique_ptr<RGBAImage> image) {
+	images[ident] = std::move(image);
 	height = -1;
 	width = -1;
 }
@@ -314,7 +314,7 @@ RGBAImage *RGBAImageSet::Get(int ident) {
 }
 
 /// Give the largest height of the set.
-int RGBAImageSet::GetHeight() const {
+int RGBAImageSet::GetHeight() const noexcept {
 	if (height < 0) {
 		for (const std::pair<const int, std::unique_ptr<RGBAImage>> &image : images) {
 			if (height < image.second->GetHeight()) {
@@ -326,7 +326,7 @@ int RGBAImageSet::GetHeight() const {
 }
 
 /// Give the largest width of the set.
-int RGBAImageSet::GetWidth() const {
+int RGBAImageSet::GetWidth() const noexcept {
 	if (width < 0) {
 		for (const std::pair<const int, std::unique_ptr<RGBAImage>> &image : images) {
 			if (width < image.second->GetWidth()) {
