@@ -91,7 +91,7 @@ static gboolean on_window_delete_event(GtkWidget *widget, GdkEvent *event, gpoin
 
 void on_new1_activate(GtkMenuItem *menuitem, gpointer user_data)
 {
-	document_new_file(NULL, NULL, NULL);
+	keybindings_send_command(GEANY_KEY_GROUP_FILE, GEANY_KEYS_FILE_NEW);
 }
 
 
@@ -437,6 +437,44 @@ void on_toolbutton_search_clicked(GtkAction *action, gpointer user_data)
 }
 
 
+void on_entry_tagfilter_changed(GtkAction *action, gpointer user_data)
+{
+	GeanyDocument *doc = document_get_current();
+	GtkEntry *filter_entry;
+
+	if (!doc)
+		return;
+
+	filter_entry = GTK_ENTRY(ui_lookup_widget(main_widgets.window, "entry_tagfilter"));
+	g_free(doc->priv->tag_filter);
+	doc->priv->tag_filter = g_strdup(gtk_entry_get_text(filter_entry));
+
+	/* make sure the tree is fully re-created so it appears correctly
+	 * after applying filter */
+	if (doc->priv->tag_store)
+		gtk_tree_store_clear(doc->priv->tag_store);
+	sidebar_update_tag_list(doc, TRUE);
+}
+
+
+void on_entry_tagfilter_icon_press(GtkEntry *entry, GtkEntryIconPosition icon_pos, GdkEvent *event, gpointer user_data)
+{
+	if (event->button.button == 1)
+		gtk_entry_set_text(entry, "");
+}
+
+
+void on_entry_tagfilter_activate(GtkEntry *entry, gpointer user_data)
+{
+	GeanyDocument *doc = document_get_current();
+
+	if (!doc)
+		return;
+
+	gtk_widget_grab_focus(doc->priv->tag_tree);
+}
+
+
 /* hides toolbar from toolbar popup menu */
 static void on_hide_toolbar1_activate(GtkMenuItem *menuitem, gpointer user_data)
 {
@@ -491,6 +529,9 @@ static void on_notebook1_switch_page_after(GtkNotebook *notebook, gpointer page,
 
 	if (doc != NULL)
 	{
+		GtkEntry *filter_entry = GTK_ENTRY(ui_lookup_widget(main_widgets.window, "entry_tagfilter"));
+		const gchar *entry_text = gtk_entry_get_text(filter_entry);
+
 		sidebar_select_openfiles_item(doc);
 		ui_save_buttons_toggle(doc->changed);
 		ui_set_window_title(doc);
@@ -498,7 +539,13 @@ static void on_notebook1_switch_page_after(GtkNotebook *notebook, gpointer page,
 		ui_update_popup_reundo_items(doc);
 		ui_document_show_hide(doc); /* update the document menu */
 		build_menu_update(doc);
-		sidebar_update_tag_list(doc, FALSE);
+		if (g_strcmp0(entry_text, doc->priv->tag_filter) != 0)
+		{
+			/* calls sidebar_update_tag_list() in on_entry_tagfilter_changed() */
+			gtk_entry_set_text(filter_entry, doc->priv->tag_filter);
+		}
+		else
+			sidebar_update_tag_list(doc, TRUE);
 		document_highlight_tags(doc);
 
 		document_check_disk_status(doc, TRUE);
@@ -587,7 +634,7 @@ gboolean toolbar_popup_menu(GtkWidget *widget, GdkEventButton *event, gpointer u
 {
 	if (event->button == 3)
 	{
-		gtk_menu_popup(GTK_MENU(ui_widgets.toolbar_menu), NULL, NULL, NULL, NULL, event->button, event->time);
+		gtk_menu_popup_at_pointer(GTK_MENU(ui_widgets.toolbar_menu), (GdkEvent *) event);
 		return TRUE;
 	}
 	return FALSE;
@@ -911,21 +958,6 @@ void on_find_in_files1_activate(GtkMenuItem *menuitem, gpointer user_data)
 }
 
 
-static void get_line_and_offset_from_text(const gchar *text, gint *line_no, gint *offset)
-{
-	if (*text == '+' || *text == '-')
-	{
-		*line_no = atoi(text + 1);
-		*offset = (*text == '+') ? 1 : -1;
-	}
-	else
-	{
-		*line_no = atoi(text) - 1;
-		*offset = 0;
-	}
-}
-
-
 void on_go_to_line_activate(GtkMenuItem *menuitem, gpointer user_data)
 {
 	static gchar value[16] = "";
@@ -936,18 +968,10 @@ void on_go_to_line_activate(GtkMenuItem *menuitem, gpointer user_data)
 		_("Enter the line you want to go to:"), value);
 	if (result != NULL)
 	{
-		GeanyDocument *doc = document_get_current();
-		gint offset;
-		gint line_no;
+		on_toolbutton_goto_entry_activate(NULL, result, NULL);
 
-		g_return_if_fail(doc != NULL);
-
-		get_line_and_offset_from_text(result, &line_no, &offset);
-		if (! editor_goto_line(doc->editor, line_no, offset))
-			utils_beep();
 		/* remember value for future calls */
 		g_snprintf(value, sizeof(value), "%s", result);
-
 		g_free(result);
 	}
 }
@@ -956,12 +980,11 @@ void on_go_to_line_activate(GtkMenuItem *menuitem, gpointer user_data)
 void on_toolbutton_goto_entry_activate(GtkAction *action, const gchar *text, gpointer user_data)
 {
 	GeanyDocument *doc = document_get_current();
-	gint offset;
-	gint line_no;
-
 	g_return_if_fail(doc != NULL);
 
-	get_line_and_offset_from_text(text, &line_no, &offset);
+	gint line_no = atoi(text);
+	gboolean offset = (*text == '+' || *text == '-');
+
 	if (! editor_goto_line(doc->editor, line_no, offset))
 		utils_beep();
 	else
@@ -1236,7 +1259,7 @@ void on_menu_select_all1_activate(GtkMenuItem *menuitem, gpointer user_data)
 	}
 	/* special case for Select All in the VTE widget */
 #ifdef HAVE_VTE
-	else if (vte_info.have_vte && focusw == vc->vte)
+	else if (vte_info.have_vte && focusw == vte_config.vte)
 	{
 		vte_select_all();
 	}
@@ -1364,7 +1387,13 @@ void on_previous_message1_activate(GtkMenuItem *menuitem, gpointer user_data)
 
 void on_project_new1_activate(GtkMenuItem *menuitem, gpointer user_data)
 {
-	project_new();
+	project_new(FALSE);
+}
+
+
+void on_project_new_from_folder1_activate(GtkMenuItem *menuitem, gpointer user_data)
+{
+	project_new(TRUE);
 }
 
 
@@ -1873,6 +1902,12 @@ static void on_go_to_previous_marker1_activate(GtkMenuItem *menuitem, gpointer u
 }
 
 
+static void on_join_lines1_activate(GtkMenuItem *menuitem, gpointer user_data)
+{
+	keybindings_send_command(GEANY_KEY_GROUP_FORMAT, GEANY_KEYS_FORMAT_JOINLINES);
+}
+
+
 static void on_reflow_lines_block1_activate(GtkMenuItem *menuitem, gpointer user_data)
 {
 	keybindings_send_command(GEANY_KEY_GROUP_FORMAT, GEANY_KEYS_FORMAT_REFLOWPARAGRAPH);
