@@ -20,7 +20,7 @@
 
 /*
  * A GtkLabel subclass that can wrap to any width, unlike GtkLabel which has a fixed wrap point.
- * (inspired by libview's WrapLabel, http://view.sourceforge.net)
+ * (inspired by libview's WrapLabel, https://view.sourceforge.net/)
  */
 
 #ifdef HAVE_CONFIG_H
@@ -48,7 +48,6 @@ struct _GeanyWrapLabel
 };
 
 
-#if GTK_CHECK_VERSION(3, 0, 0)
 static gboolean geany_wrap_label_draw(GtkWidget *widget, cairo_t *cr);
 static void geany_wrap_label_get_preferred_width (GtkWidget *widget,
 		gint *minimal_width, gint *natural_width);
@@ -59,10 +58,6 @@ static void geany_wrap_label_get_preferred_width_for_height (GtkWidget *widget,
 static void geany_wrap_label_get_preferred_height_for_width (GtkWidget *widget,
 		gint width, gint *minimal_height, gint *natural_height);
 static GtkSizeRequestMode geany_wrap_label_get_request_mode(GtkWidget *widget);
-#else
-static gboolean geany_wrap_label_expose		(GtkWidget *widget, GdkEventExpose *event);
-static void geany_wrap_label_size_request	(GtkWidget *widget, GtkRequisition *req);
-#endif
 static void geany_wrap_label_size_allocate	(GtkWidget *widget, GtkAllocation *alloc);
 static void geany_wrap_label_set_wrap_width	(GtkWidget *widget, gint width);
 static void geany_wrap_label_label_notify	(GObject *object, GParamSpec *pspec, gpointer data);
@@ -75,17 +70,12 @@ static void geany_wrap_label_class_init(GeanyWrapLabelClass *klass)
 	GtkWidgetClass *widget_class = GTK_WIDGET_CLASS(klass);
 
 	widget_class->size_allocate = geany_wrap_label_size_allocate;
-#if GTK_CHECK_VERSION(3, 0, 0)
 	widget_class->draw = geany_wrap_label_draw;
 	widget_class->get_preferred_width = geany_wrap_label_get_preferred_width;
 	widget_class->get_preferred_width_for_height = geany_wrap_label_get_preferred_width_for_height;
 	widget_class->get_preferred_height = geany_wrap_label_get_preferred_height;
 	widget_class->get_preferred_height_for_width = geany_wrap_label_get_preferred_height_for_width;
 	widget_class->get_request_mode = geany_wrap_label_get_request_mode;
-#else
-	widget_class->size_request = geany_wrap_label_size_request;
-	widget_class->expose_event = geany_wrap_label_expose;
-#endif
 
 	g_type_class_add_private(klass, sizeof (GeanyWrapLabelPrivate));
 }
@@ -140,7 +130,6 @@ static void geany_wrap_label_label_notify(GObject *object, GParamSpec *pspec, gp
 }
 
 
-#if GTK_CHECK_VERSION(3, 0, 0)
 /* makes sure the layout is setup for rendering and chains to parent renderer */
 static gboolean geany_wrap_label_draw(GtkWidget *widget, cairo_t *cr)
 {
@@ -199,48 +188,22 @@ static GtkSizeRequestMode geany_wrap_label_get_request_mode(GtkWidget *widget)
 	return GTK_SIZE_REQUEST_WIDTH_FOR_HEIGHT;
 }
 
-#else /* GTK3 */
-
-/* makes sure the layout is setup for rendering and chains to parent renderer */
-static gboolean geany_wrap_label_expose(GtkWidget *widget, GdkEventExpose *event)
-{
-	GeanyWrapLabel *self = GEANY_WRAP_LABEL(widget);
-	PangoLayout *layout = gtk_label_get_layout(GTK_LABEL(widget));
-
-	pango_layout_set_width(layout, self->priv->wrap_width * PANGO_SCALE);
-	pango_layout_set_wrap(layout, PANGO_WRAP_WORD_CHAR);
-
-	return (* GTK_WIDGET_CLASS(geany_wrap_label_parent_class)->expose_event)(widget, event);
-}
-
-
-/* Forces the height to be the size necessary for the Pango layout, while allowing the
- * width to be flexible. */
-static void geany_wrap_label_size_request(GtkWidget *widget, GtkRequisition *req)
-{
-	req->width = 0;
-	req->height = GEANY_WRAP_LABEL(widget)->priv->wrap_height;
-}
-#endif /* GTK3 */
-
 
 /* Sets the wrap width to the width allocated to us. */
 static void geany_wrap_label_size_allocate(GtkWidget *widget, GtkAllocation *alloc)
 {
+	GtkWidget *parent;
+
 	(* GTK_WIDGET_CLASS(geany_wrap_label_parent_class)->size_allocate)(widget, alloc);
 
 	geany_wrap_label_set_wrap_width(widget, alloc->width);
 
-#if GTK_CHECK_VERSION(3, 0, 0)
-{
-	/* ask the parent to recompute our size, because it seems GTK3 size
+	/* ask the parent to recompute our size, because it seems GTK size
 	 * caching is too aggressive */
-	GtkWidget *parent = gtk_widget_get_parent(widget);
+	parent = gtk_widget_get_parent(widget);
 	if (GTK_IS_CONTAINER(parent))
 		gtk_container_check_resize(GTK_CONTAINER(parent));
 }
-#endif
-}
 
 
 GtkWidget *geany_wrap_label_new(const gchar *text)
