@@ -1000,7 +1000,9 @@ void highlighting_init_styles(guint filetype_idx, GKeyFile *config, GKeyFile *co
 	{
 		init_styleset_case(ABAQUS);
 		init_styleset_case(ADA);
+		init_styleset_case(ASCIIDOC);
 		init_styleset_case(ASM);
+		init_styleset_case(AU3);
 		init_styleset_case(BASIC);
 		init_styleset_case(BATCH);
 		init_styleset_case(C);
@@ -1015,10 +1017,10 @@ void highlighting_init_styles(guint filetype_idx, GKeyFile *config, GKeyFile *co
 		init_styleset_case(LISP);
 		init_styleset_case(ERLANG);
 		init_styleset_case(DOCBOOK);
-		init_styleset_case(FERITE);
 		init_styleset_case(F77);
 		init_styleset_case(FORTH);
 		init_styleset_case(FORTRAN);
+		init_styleset_case(GDSCRIPT);
 		init_styleset_case(GO);
 		init_styleset_case(HASKELL);
 		init_styleset_case(HAXE);
@@ -1090,7 +1092,9 @@ void highlighting_set_styles(ScintillaObject *sci, GeanyFiletype *ft)
 	{
 		styleset_case(ABAQUS);
 		styleset_case(ADA);
+		styleset_case(ASCIIDOC);
 		styleset_case(ASM);
+		styleset_case(AU3);
 		styleset_case(BASIC);
 		styleset_case(BATCH);
 		styleset_case(C);
@@ -1105,10 +1109,10 @@ void highlighting_set_styles(ScintillaObject *sci, GeanyFiletype *ft)
 		styleset_case(LISP);
 		styleset_case(ERLANG);
 		styleset_case(DOCBOOK);
-		styleset_case(FERITE);
 		styleset_case(F77);
 		styleset_case(FORTH);
 		styleset_case(FORTRAN);
+		styleset_case(GDSCRIPT);
 		styleset_case(GO);
 		styleset_case(HASKELL);
 		styleset_case(HAXE);
@@ -1168,7 +1172,7 @@ void highlighting_set_styles(ScintillaObject *sci, GeanyFiletype *ft)
  * (e.g. by by opening a file of this type), it will be initialised. The returned pointer is
  * owned by Geany and must not be freed.
  * @param ft_id Filetype ID, e.g. @c GEANY_FILETYPES_DIFF.
- * @param style_id A Scintilla lexer style, e.g. @c SCE_DIFF_ADDED. See scintilla/include/SciLexer.h.
+ * @param style_id A Scintilla lexer style, e.g. @c SCE_DIFF_ADDED. See scintilla/lexilla/include/SciLexer.h.
  * @return A pointer to the style struct.
  * @see Scintilla messages @c SCI_STYLEGETFORE, etc, for use with scintilla_send_message(). */
 GEANY_API_SYMBOL
@@ -1436,6 +1440,13 @@ gboolean highlighting_is_string_style(gint lexer, gint style)
 				style == SCE_P_FTRIPLEDOUBLE ||
 				style == SCE_P_STRINGEOL);
 
+		case SCLEX_GDSCRIPT:
+			return (style == SCE_GD_STRING ||
+				style == SCE_GD_TRIPLE ||
+				style == SCE_GD_TRIPLEDOUBLE ||
+				style == SCE_GD_CHARACTER ||
+				style == SCE_GD_STRINGEOL);
+
 		case SCLEX_F77:
 		case SCLEX_FORTRAN:
 			return (style == SCE_F_STRING1 ||
@@ -1633,6 +1644,9 @@ gboolean highlighting_is_string_style(gint lexer, gint style)
 		case SCLEX_YAML:
 			/* there is no string type in those lexers, listing here just for completeness */
 			return FALSE;
+
+		case SCLEX_AU3:
+			return (style == SCE_AU3_STRING);
 	}
 	return FALSE;
 }
@@ -1855,6 +1869,14 @@ gboolean highlighting_is_comment_style(gint lexer, gint style)
 		case SCLEX_MARKDOWN:
 			/* there is no comment type in those lexers, listing here just for completeness */
 			return FALSE;
+			
+		case SCLEX_GDSCRIPT:
+			return (style == SCE_GD_COMMENTLINE ||
+				style == SCE_GD_COMMENTBLOCK);
+
+		case SCLEX_AU3:
+			return (style == SCE_AU3_COMMENT ||
+				style == SCE_AU3_COMMENTBLOCK);
 	}
 	return FALSE;
 }
