@@ -133,6 +133,39 @@ static const HLKeyword highlighting_keywords_AS[] =
 #define highlighting_properties_AS	highlighting_properties_C
 
 
+/* Asccidoc */
+#define highlighting_lexer_ASCIIDOC			SCLEX_ASCIIDOC
+static const HLStyle highlighting_styles_ASCIIDOC[] =
+{
+	{ SCE_ASCIIDOC_DEFAULT,		"default",		FALSE },
+	{ SCE_ASCIIDOC_STRONG1,		"strong",		FALSE },
+	{ SCE_ASCIIDOC_STRONG2,		"strong",		FALSE },
+	{ SCE_ASCIIDOC_EM1,			"emphasis",		FALSE },
+	{ SCE_ASCIIDOC_EM2,			"emphasis",		FALSE },
+	{ SCE_ASCIIDOC_HEADER1,		"header1",		FALSE },
+	{ SCE_ASCIIDOC_HEADER2,		"header2",		FALSE },
+	{ SCE_ASCIIDOC_HEADER3,		"header3",		FALSE },
+	{ SCE_ASCIIDOC_HEADER4,		"header4",		FALSE },
+	{ SCE_ASCIIDOC_HEADER5,		"header5",		FALSE },
+	{ SCE_ASCIIDOC_HEADER6,		"header6",		FALSE },
+	{ SCE_ASCIIDOC_ULIST_ITEM,	"ulist_item",	FALSE },
+	{ SCE_ASCIIDOC_OLIST_ITEM,	"olist_item",	FALSE },
+	{ SCE_ASCIIDOC_BLOCKQUOTE,	"blockquote",	FALSE },
+	{ SCE_ASCIIDOC_LINK,		"link",			FALSE },
+	{ SCE_ASCIIDOC_CODEBK,		"code",			FALSE },
+	{ SCE_ASCIIDOC_PASSBK,		"passthrough",	FALSE },
+	{ SCE_ASCIIDOC_COMMENT,		"comment",		FALSE },
+	{ SCE_ASCIIDOC_COMMENTBK,	"comment",		FALSE },
+	{ SCE_ASCIIDOC_LITERAL,		"literal",		FALSE },
+	{ SCE_ASCIIDOC_LITERALBK,	"literal",		FALSE },
+	{ SCE_ASCIIDOC_ATTRIB,		"attrib",		FALSE },
+	{ SCE_ASCIIDOC_ATTRIBVAL,	"attribval",	FALSE },
+	{ SCE_ASCIIDOC_MACRO,		"macro",		FALSE }
+};
+#define highlighting_keywords_ASCIIDOC		EMPTY_KEYWORDS
+#define highlighting_properties_ASCIIDOC	EMPTY_PROPERTIES
+
+
 /* ASM */
 #define highlighting_lexer_ASM			SCLEX_ASM
 static const HLStyle highlighting_styles_ASM[] =
@@ -388,7 +421,7 @@ static const HLStyle highlighting_styles_CSS[] =
 	{ SCE_CSS_EXTENDED_IDENTIFIER,		"extended_identifier",		FALSE },
 	{ SCE_CSS_EXTENDED_PSEUDOCLASS,		"extended_pseudoclass",		FALSE },
 	{ SCE_CSS_EXTENDED_PSEUDOELEMENT,	"extended_pseudoelement",	FALSE },
-	{ SCE_CSS_MEDIA,					"media",					FALSE }
+	{ SCE_CSS_GROUP_RULE,				"group_rule",				FALSE }
 };
 static const HLKeyword highlighting_keywords_CSS[] =
 {
@@ -606,18 +639,6 @@ static const HLKeyword highlighting_keywords_F77[] =
 #define highlighting_properties_F77		EMPTY_PROPERTIES
 
 
-/* Ferite */
-#define highlighting_lexer_FERITE		SCLEX_CPP
-#define highlighting_styles_FERITE		highlighting_styles_C
-static const HLKeyword highlighting_keywords_FERITE[] =
-{
-	{ 0, "primary",		FALSE },
-	{ 1, "types",		FALSE },
-	{ 2, "docComment",	FALSE }
-};
-#define highlighting_properties_FERITE	highlighting_properties_C
-
-
 /* Forth */
 #define highlighting_lexer_FORTH		SCLEX_FORTH
 static const HLStyle highlighting_styles_FORTH[] =
@@ -655,6 +676,36 @@ static const HLKeyword highlighting_keywords_FORTH[] =
 #define highlighting_properties_FORTRAN		highlighting_properties_F77
 
 
+/* GDScript */
+#define highlighting_lexer_GDSCRIPT		SCLEX_GDSCRIPT
+static const HLStyle highlighting_styles_GDSCRIPT[] =
+{
+	{ SCE_GD_DEFAULT,		"default",			FALSE },
+	{ SCE_GD_COMMENTLINE,	"commentline",		FALSE },
+	{ SCE_GD_NUMBER,		"number",			FALSE },
+	{ SCE_GD_STRING,		"string",			FALSE },
+	{ SCE_GD_CHARACTER,		"character",		FALSE },
+	{ SCE_GD_WORD,			"word",				FALSE },
+	{ SCE_GD_TRIPLE,		"triple",			FALSE },
+	{ SCE_GD_TRIPLEDOUBLE,	"tripledouble",		FALSE },
+	{ SCE_GD_CLASSNAME,		"classname",		FALSE },
+	{ SCE_GD_FUNCNAME,		"funcname",			FALSE },
+	{ SCE_GD_OPERATOR,		"operator",			FALSE },
+	{ SCE_GD_IDENTIFIER,	"identifier",		FALSE },
+	{ SCE_GD_COMMENTBLOCK,	"commentblock",		FALSE },
+	{ SCE_GD_STRINGEOL,		"stringeol",		FALSE },
+	{ SCE_GD_WORD2,			"word2",			FALSE },
+	{ SCE_GD_ANNOTATION,	"annotation",		FALSE },
+	{ SCE_GD_NODEPATH,		"notepath",			FALSE }
+};
+static const HLKeyword highlighting_keywords_GDSCRIPT[] =
+{
+	{ 0, "primary",		FALSE },
+	{ 1, "identifiers",	FALSE }
+};
+#define highlighting_properties_GDSCRIPT	EMPTY_PROPERTIES
+
+
 /* Go */
 #define highlighting_lexer_GO		SCLEX_CPP
 #define highlighting_styles_GO		highlighting_styles_C
@@ -1299,7 +1350,8 @@ static const HLStyle highlighting_styles_PYTHON[] =
 	{ SCE_P_FCHARACTER,		"fcharacter",		FALSE },
 	{ SCE_P_FTRIPLE,		"ftriple",			FALSE },
 	{ SCE_P_FTRIPLEDOUBLE,	"ftripledouble",	FALSE },
-	{ SCE_P_DECORATOR,		"decorator",		FALSE }
+	{ SCE_P_DECORATOR,		"decorator",		FALSE },
+	{ SCE_P_ATTRIBUTE,		"attribute",		FALSE }
 };
 static const HLKeyword highlighting_keywords_PYTHON[] =
 {
@@ -1313,18 +1365,22 @@ static const HLKeyword highlighting_keywords_PYTHON[] =
 #define highlighting_lexer_R		SCLEX_R
 static const HLStyle highlighting_styles_R[] =
 {
-	{ SCE_R_DEFAULT,	"default",		FALSE },
-	{ SCE_R_COMMENT,	"comment",		FALSE },
-	{ SCE_R_KWORD,		"kword",		FALSE },
-	{ SCE_R_OPERATOR,	"operator",		FALSE },
-	{ SCE_R_BASEKWORD,	"basekword",	FALSE },
-	{ SCE_R_OTHERKWORD,	"otherkword",	FALSE },
-	{ SCE_R_NUMBER,		"number",		FALSE },
-	{ SCE_R_STRING,		"string",		FALSE },
-	{ SCE_R_STRING2,	"string2",		FALSE },
-	{ SCE_R_IDENTIFIER,	"identifier",	FALSE },
-	{ SCE_R_INFIX,		"infix",		FALSE },
-	{ SCE_R_INFIXEOL,	"infixeol",		FALSE }
+	{ SCE_R_DEFAULT,		"default",			FALSE },
+	{ SCE_R_COMMENT,		"comment",			FALSE },
+	{ SCE_R_KWORD,			"kword",			FALSE },
+	{ SCE_R_OPERATOR,		"operator",			FALSE },
+	{ SCE_R_BASEKWORD,		"basekword",		FALSE },
+	{ SCE_R_OTHERKWORD,		"otherkword",		FALSE },
+	{ SCE_R_NUMBER,			"number",			FALSE },
+	{ SCE_R_STRING,			"string",			FALSE },
+	{ SCE_R_STRING2,		"string2",			FALSE },
+	{ SCE_R_IDENTIFIER,		"identifier",		FALSE },
+	{ SCE_R_INFIX,			"infix",			FALSE },
+	{ SCE_R_INFIXEOL,		"infixeol",			FALSE },
+	{ SCE_R_BACKTICKS,		"backticks",		FALSE },
+	{ SCE_R_RAWSTRING,		"stringraw",		FALSE },
+	{ SCE_R_RAWSTRING2,		"stringraw",		FALSE },
+	{ SCE_R_ESCAPESEQUENCE,	"escapesequence",	FALSE }
 };
 static const HLKeyword highlighting_keywords_R[] =
 {
@@ -1370,6 +1426,10 @@ static const HLStyle highlighting_styles_RUBY[] =
 	{ SCE_RB_STRING_QX,		"string_qx",		FALSE },
 	{ SCE_RB_STRING_QR,		"string_qr",		FALSE },
 	{ SCE_RB_STRING_QW,		"string_qw",		FALSE },
+	{ SCE_RB_STRING_W,		"string_qw",		FALSE },
+	{ SCE_RB_STRING_QI,		"symbol",			FALSE },
+	{ SCE_RB_STRING_QS,		"symbol",			FALSE },
+	{ SCE_RB_STRING_I,		"symbol",			FALSE },
 	{ SCE_RB_UPPER_BOUND,	"upper_bound",		FALSE },
 	{ SCE_RB_ERROR,			"error",			FALSE },
 	{ SCE_RB_POD,			"pod",				FALSE }
@@ -1690,6 +1750,41 @@ static const HLKeyword highlighting_keywords_YAML[] =
 #define highlighting_keywords_ZEPHIR	highlighting_keywords_PHP
 #define highlighting_properties_ZEPHIR	highlighting_properties_PHP
 
+
+/* AutoIt */
+#define highlighting_lexer_AU3			SCLEX_AU3
+static const HLStyle highlighting_styles_AU3[] =
+{
+	{ SCE_AU3_DEFAULT,		"default",		FALSE},
+	{ SCE_AU3_COMMENT,		"comment",		FALSE},
+	{ SCE_AU3_COMMENTBLOCK,		"commentblock",		FALSE},
+	{ SCE_AU3_NUMBER,		"number",		FALSE},
+	{ SCE_AU3_FUNCTION,		"function",		FALSE},
+	{ SCE_AU3_KEYWORD,		"keyword",		FALSE},
+	{ SCE_AU3_MACRO,		"macro",		FALSE},
+	{ SCE_AU3_STRING,		"string",		FALSE},
+	{ SCE_AU3_OPERATOR,		"operator",		FALSE},
+	{ SCE_AU3_VARIABLE,		"variable",		FALSE},
+	{ SCE_AU3_SENT,			"sent",			FALSE},
+	{ SCE_AU3_PREPROCESSOR,		"preprocessor",		FALSE},
+	{ SCE_AU3_SPECIAL,		"special",		FALSE},
+	{ SCE_AU3_EXPAND,		"expand",		FALSE},
+	{ SCE_AU3_COMOBJ,		"comobj",		FALSE},
+	{ SCE_AU3_UDF,			"udf",			FALSE}
+};
+static const HLKeyword highlighting_keywords_AU3[] =
+{
+	{ 0, "keywords",	FALSE },
+	{ 1, "functions",	FALSE },
+	{ 2, "macros",		FALSE },
+	{ 3, "sent",		FALSE },
+	{ 4, "preprocessor",	FALSE },
+	{ 5, "special",		FALSE },
+	{ 6, "expand",		FALSE },
+	{ 7, "udf",		FALSE }
+};
+#define highlighting_properties_AU3		EMPTY_PROPERTIES
+
 G_END_DECLS
 
 #endif /* GEANY_HIGHLIGHTING_MAPPINGS_H */
