@@ -46,6 +46,7 @@
 #include "printing.h"
 #include "project.h"
 #include "sciwrappers.h"
+#include "sidebar.h"
 #include "socket.h"
 #include "stash.h"
 #include "support.h"
@@ -66,6 +67,9 @@
 #include <unistd.h>
 #endif
 
+/* define the configuration filenames */
+#define PREFS_FILE						"geany.conf"
+#define SESSION_FILE					"session.conf"
 
 /* some default settings which are used at the very first start of Geany to fill
  * the configuration file */
@@ -87,7 +91,8 @@
 #define GEANY_DEFAULT_FONT_MSG_WINDOW	"Menlo Medium 12"
 #define GEANY_DEFAULT_FONT_EDITOR		"Menlo Medium 12"
 #else
-#define GEANY_DEFAULT_TOOLS_BROWSER		"firefox"
+/* Browser chosen by GTK */
+#define GEANY_DEFAULT_TOOLS_BROWSER		""
 #define GEANY_DEFAULT_FONT_SYMBOL_LIST	"Sans 9"
 #define GEANY_DEFAULT_FONT_MSG_WINDOW	"Monospace 9"
 #define GEANY_DEFAULT_FONT_EDITOR		"Monospace 10"
@@ -103,16 +108,59 @@
 
 static gchar *scribble_text = NULL;
 static gint scribble_pos = -1;
-static GPtrArray *session_files = NULL;
+static GPtrArray *default_session_files = NULL;
 static gint session_notebook_page;
 static gint hpan_position;
 static gint vpan_position;
-static guint document_list_update_idle_func_id = 0;
 static const gchar atomic_file_saving_key[] = "use_atomic_file_saving";
 
-static GPtrArray *keyfile_groups = NULL;
+typedef enum
+{
+	PREFS,
+	SESSION,
 
-GPtrArray *pref_groups = NULL;
+	MAX_PAYLOAD
+} ConfigPayload;
+
+/* Fallback only used when loading config. Saving config should always write the real file. */
+static gchar *get_keyfile_for_payload(ConfigPayload payload)
+{
+	static gboolean logged;
+	gchar *file;
+
+	switch (payload)
+	{
+		case PREFS:
+			file = g_build_filename(app->configdir, PREFS_FILE, NULL);
+			if (! g_file_test(file, G_FILE_TEST_IS_REGULAR))
+			{	/* config file does not (yet) exist, so try to load a global config
+				 * file which may be created by distributors */
+				g_message("No user config file found, trying to use global configuration.");
+				g_free(file);
+				file = g_build_filename(app->datadir, PREFS_FILE, NULL);
+			}
+			return file;
+		case SESSION:
+			file = g_build_filename(app->configdir, SESSION_FILE, NULL);
+			if (! g_file_test(file, G_FILE_TEST_IS_REGULAR))
+			{
+				if (!logged)
+				{
+					g_message("No user session file found, trying to use configuration file.");
+					logged = TRUE;
+				}
+				g_free(file);
+				file = g_build_filename(app->configdir, PREFS_FILE, NULL);
+			}
+			return file;
+		case MAX_PAYLOAD:
+			g_assert_not_reached();
+	}
+	return NULL;
+}
+
+static GPtrArray *keyfile_groups[MAX_PAYLOAD];
+GPtrArray *pref_groups;
 
 static struct
 {
@@ -122,12 +170,11 @@ static struct
 }
 build_menu_prefs;
 
-
 /* The group will be free'd on quitting.
  * @param for_prefs_dialog is whether the group also has Prefs dialog items. */
 void configuration_add_pref_group(struct StashGroup *group, gboolean for_prefs_dialog)
 {
-	g_ptr_array_add(keyfile_groups, group);
+	g_ptr_array_add(keyfile_groups[PREFS], group);
 
 	if (for_prefs_dialog)
 		g_ptr_array_add(pref_groups, group);
@@ -144,6 +191,23 @@ void configuration_add_various_pref_group(struct StashGroup *group,
 }
 
 
+/* The group will be free'd on quitting.
+ *
+ * @a for_prefs_dialog is typically @c FALSE as session configuration is not
+ * well suited for the Preferences dialog. Probably only existing prefs that
+ * migrated to session data should set this to @c TRUE.
+ *
+ * @param for_prefs_dialog is whether the group also has Prefs dialog items.
+ */
+void configuration_add_session_group(struct StashGroup *group, gboolean for_prefs_dialog)
+{
+	g_ptr_array_add(keyfile_groups[SESSION], group);
+
+	if (for_prefs_dialog)
+		g_ptr_array_add(pref_groups, group);
+}
+
+
 static void init_pref_groups(void)
 {
 	StashGroup *group;
@@ -161,6 +225,7 @@ static void init_pref_groups(void)
 	stash_group_add_toggle_button(group, &file_prefs.tab_close_switch_to_mru,
 		"tab_close_switch_to_mru", FALSE, "check_tab_close_switch_to_mru");
 	stash_group_add_integer(group, &interface_prefs.tab_pos_sidebar, "tab_pos_sidebar", GTK_POS_TOP);
+	stash_group_add_integer(group, &interface_prefs.openfiles_path_mode, "openfiles_path_mode", -1);
 	stash_group_add_radio_buttons(group, &interface_prefs.sidebar_pos,
 		"sidebar_pos", GTK_POS_LEFT,
 		"radio_sidebar_left", GTK_POS_LEFT,
@@ -191,6 +256,8 @@ static void init_pref_groups(void)
 		"detect_indent_width", FALSE, "check_detect_indent_width");
 	stash_group_add_toggle_button(group, &editor_prefs.use_tab_to_indent,
 		"use_tab_to_indent", TRUE, "check_tab_key_indents");
+	stash_group_add_toggle_button(group, &editor_prefs.backspace_unindent,
+		"backspace_unindent", TRUE, "check_backspace_unindent");
 	stash_group_add_spin_button_integer(group, &editor_prefs.indentation->width,
 		"pref_editor_tab_width", 4, "spin_indent_width");
 	stash_group_add_combo_box(group, (gint*)(void*)&editor_prefs.indentation->auto_indent_mode,
@@ -207,6 +274,10 @@ static void init_pref_groups(void)
 		"radio_virtualspace_selection", GEANY_VIRTUAL_SPACE_SELECTION,
 		"radio_virtualspace_always", GEANY_VIRTUAL_SPACE_ALWAYS,
 		NULL);
+	stash_group_add_toggle_button(group, &editor_prefs.change_history_markers,
+		"change_history_markers", FALSE, "check_change_history_markers");
+	stash_group_add_toggle_button(group, &editor_prefs.change_history_indicators,
+		"change_history_indicators", FALSE, "check_change_history_indicators");
 	stash_group_add_toggle_button(group, &editor_prefs.autocomplete_doc_words,
 		"autocomplete_doc_words", FALSE, "check_autocomplete_doc_words");
 	stash_group_add_toggle_button(group, &editor_prefs.completion_drops_rest_of_word,
@@ -283,6 +354,17 @@ static void init_pref_groups(void)
 		"socket_remote_cmd_port", SOCKET_WINDOWS_REMOTE_CMD_PORT);
 #endif
 
+#ifdef HAVE_VTE
+	/* various VTE prefs. */
+	group = stash_group_new("VTE");
+	configuration_add_various_pref_group(group, "terminal");
+
+	stash_group_add_string(group, &vte_config.send_cmd_prefix,
+		"send_cmd_prefix", "");
+	stash_group_add_boolean(group, &vte_config.send_selection_unsafe,
+		"send_selection_unsafe", FALSE);
+#endif
+
 	/* Note: Interface-related various prefs are in ui_init_prefs() */
 
 	/* various build-menu prefs */
@@ -306,12 +388,12 @@ typedef enum SettingAction
 }
 SettingAction;
 
-static void settings_action(GKeyFile *config, SettingAction action)
+static void settings_action(GKeyFile *config, SettingAction action, ConfigPayload payload)
 {
 	guint i;
 	StashGroup *group;
 
-	foreach_ptr_array(group, i, keyfile_groups)
+	foreach_ptr_array(group, i, keyfile_groups[payload])
 	{
 		switch (action)
 		{
@@ -437,14 +519,10 @@ void configuration_save_session_files(GKeyFile *config)
 
 static void save_dialog_prefs(GKeyFile *config)
 {
-	/* new settings should be added in init_pref_groups() */
-	settings_action(config, SETTING_WRITE);
-
 	/* Some of the key names are not consistent, but this is for backwards compatibility */
 
 	/* general */
 	g_key_file_set_boolean(config, PACKAGE, "pref_main_load_session", prefs.load_session);
-	g_key_file_set_boolean(config, PACKAGE, "pref_main_project_session", project_prefs.project_session);
 	g_key_file_set_boolean(config, PACKAGE, "pref_main_project_file_in_basedir", project_prefs.project_file_in_basedir);
 	g_key_file_set_boolean(config, PACKAGE, "pref_main_save_winpos", prefs.save_winpos);
 	g_key_file_set_boolean(config, PACKAGE, "pref_main_save_wingeom", prefs.save_wingeom);
@@ -466,12 +544,13 @@ static void save_dialog_prefs(GKeyFile *config)
 	g_key_file_set_boolean(config, PACKAGE, "tab_order_beside", file_prefs.tab_order_beside);
 	g_key_file_set_integer(config, PACKAGE, "tab_pos_editor", interface_prefs.tab_pos_editor);
 	g_key_file_set_integer(config, PACKAGE, "tab_pos_msgwin", interface_prefs.tab_pos_msgwin);
-	g_key_file_set_boolean(config, PACKAGE, "use_native_windows_dialogs", interface_prefs.use_native_windows_dialogs);
+	g_key_file_set_integer(config, PACKAGE, "tab_label_length", interface_prefs.tab_label_len);
 
 	/* display */
 	g_key_file_set_boolean(config, PACKAGE, "show_indent_guide", editor_prefs.show_indent_guide);
 	g_key_file_set_boolean(config, PACKAGE, "show_white_space", editor_prefs.show_white_space);
 	g_key_file_set_boolean(config, PACKAGE, "show_line_endings", editor_prefs.show_line_endings);
+	g_key_file_set_boolean(config, PACKAGE, "show_line_endings_only_when_differ", editor_prefs.show_line_endings_only_when_differ);
 	g_key_file_set_boolean(config, PACKAGE, "show_markers_margin", editor_prefs.show_markers_margin);
 	g_key_file_set_boolean(config, PACKAGE, "show_linenumber_margin", editor_prefs.show_linenumber_margin);
 	g_key_file_set_boolean(config, PACKAGE, "long_line_enabled", editor_prefs.long_line_enabled);
@@ -551,6 +630,7 @@ static void save_dialog_prefs(GKeyFile *config)
 	g_key_file_set_boolean(config, "VTE", "load_vte", vte_info.load_vte);
 	if (vte_info.have_vte)
 	{
+		VteConfig *vc = &vte_config;
 		gchar *tmp_string;
 
 		g_key_file_set_string(config, "VTE", "font", vc->font);
@@ -582,6 +662,7 @@ static void save_ui_prefs(GKeyFile *config)
 	g_key_file_set_boolean(config, PACKAGE, "statusbar_visible", interface_prefs.statusbar_visible);
 	g_key_file_set_boolean(config, PACKAGE, "msgwindow_visible", ui_prefs.msgwindow_visible);
 	g_key_file_set_boolean(config, PACKAGE, "fullscreen", ui_prefs.fullscreen);
+	g_key_file_set_boolean(config, PACKAGE, "symbols_group_by_type", ui_prefs.symbols_group_by_type);
 	g_key_file_set_string(config, PACKAGE, "color_picker_palette", ui_prefs.color_picker_palette);
 
 	/* get the text from the scribble textview */
@@ -602,6 +683,18 @@ static void save_ui_prefs(GKeyFile *config)
 		g_key_file_set_integer(config, PACKAGE, "scribble_pos", scribble_pos);
 	}
 
+	g_key_file_set_string(config, PACKAGE, "custom_date_format", ui_prefs.custom_date_format);
+	if (ui_prefs.custom_commands != NULL)
+	{
+		g_key_file_set_string_list(config, PACKAGE, "custom_commands",
+				(const gchar**) ui_prefs.custom_commands, g_strv_length(ui_prefs.custom_commands));
+		g_key_file_set_string_list(config, PACKAGE, "custom_commands_labels",
+				(const gchar**) ui_prefs.custom_commands_labels, g_strv_length(ui_prefs.custom_commands_labels));
+	}
+}
+
+static void save_ui_session(GKeyFile *config)
+{
 	if (prefs.save_winpos || prefs.save_wingeom)
 	{
 		GdkWindowState wstate;
@@ -617,44 +710,48 @@ static void save_ui_prefs(GKeyFile *config)
 		ui_prefs.geometry[4] = (wstate & GDK_WINDOW_STATE_MAXIMIZED) ? 1 : 0;
 		g_key_file_set_integer_list(config, PACKAGE, "geometry", ui_prefs.geometry, 5);
 	}
-
-	g_key_file_set_string(config, PACKAGE, "custom_date_format", ui_prefs.custom_date_format);
-	if (ui_prefs.custom_commands != NULL)
-	{
-		g_key_file_set_string_list(config, PACKAGE, "custom_commands",
-				(const gchar**) ui_prefs.custom_commands, g_strv_length(ui_prefs.custom_commands));
-		g_key_file_set_string_list(config, PACKAGE, "custom_commands_labels",
-				(const gchar**) ui_prefs.custom_commands_labels, g_strv_length(ui_prefs.custom_commands_labels));
-	}
 }
 
-
-void configuration_save(void)
+static void write_config_file(ConfigPayload payload)
 {
 	GKeyFile *config = g_key_file_new();
-	gchar *configfile = g_build_filename(app->configdir, "geany.conf", NULL);
+	const gchar *filename;
+	gchar *configfile;
 	gchar *data;
 
+	filename = payload == SESSION ? SESSION_FILE : PREFS_FILE;
+	configfile = g_build_filename(app->configdir, filename, NULL);
 	g_key_file_load_from_file(config, configfile, G_KEY_FILE_NONE, NULL);
 
-	/* this signal can be used e.g. to prepare any settings before Stash code reads them below */
-	g_signal_emit_by_name(geany_object, "save-settings", config);
-
-	save_dialog_prefs(config);
-	save_ui_prefs(config);
-	project_save_prefs(config);	/* save project filename, etc. */
-	save_recent_files(config, ui_prefs.recent_queue, "recent_files");
-	save_recent_files(config, ui_prefs.recent_projects_queue, "recent_projects");
-
-	if (cl_options.load_session)
-		configuration_save_session_files(config);
-#ifdef HAVE_VTE
-	else if (vte_info.have_vte)
+	switch (payload)
 	{
-		vte_get_working_directory();	/* refresh vte_info.dir */
-		g_key_file_set_string(config, "VTE", "last_dir", vte_info.dir);
-	}
+		case PREFS:
+			/* this signal can be used e.g. to prepare any settings before Stash code reads them below */
+			g_signal_emit_by_name(geany_object, "save-settings", config);
+			save_dialog_prefs(config);
+			save_ui_prefs(config);
+			break;
+		case SESSION:
+			save_recent_files(config, ui_prefs.recent_queue, "recent_files");
+			save_recent_files(config, ui_prefs.recent_projects_queue, "recent_projects");
+			project_save_prefs(config);	/* save project filename, etc. */
+			save_ui_session(config);
+			if (cl_options.load_session && app->project == NULL)
+				configuration_save_session_files(config);
+#ifdef HAVE_VTE
+			else if (vte_info.have_vte)
+			{
+				vte_get_working_directory();	/* refresh vte_info.dir */
+				g_key_file_set_string(config, "VTE", "last_dir", vte_info.dir);
+			}
 #endif
+			break;
+		case MAX_PAYLOAD:
+			g_assert_not_reached();
+	}
+
+	/* new settings should be added in init_pref_groups() */
+	settings_action(config, SETTING_WRITE, payload);
 
 	/* write the file */
 	data = g_key_file_to_data(config, NULL, NULL);
@@ -665,6 +762,14 @@ void configuration_save(void)
 	g_free(configfile);
 }
 
+void configuration_save(void)
+{
+	/* save all configuration files
+	 * it is probably not very efficient to write both files every time
+	 * could be more selective about which file is saved when */
+	write_config_file(PREFS);
+	write_config_file(SESSION);
+}
 
 static void load_recent_files(GKeyFile *config, GQueue *queue, const gchar *key)
 {
@@ -685,34 +790,20 @@ static void load_recent_files(GKeyFile *config, GQueue *queue, const gchar *key)
 
 
 /*
- * Load session list from the given keyfile, and store it in the global
- * session_files variable for later file loading
+ * Load session list from the given keyfile and return an array containg the file names
  * */
-void configuration_load_session_files(GKeyFile *config, gboolean read_recent_files)
+GPtrArray *configuration_load_session_files(GKeyFile *config)
 {
 	guint i;
 	gboolean have_session_files;
 	gchar entry[16];
 	gchar **tmp_array;
 	GError *error = NULL;
+	GPtrArray *files;
 
 	session_notebook_page = utils_get_setting_integer(config, "files", "current_page", -1);
 
-	if (read_recent_files)
-	{
-		load_recent_files(config, ui_prefs.recent_queue, "recent_files");
-		load_recent_files(config, ui_prefs.recent_projects_queue, "recent_projects");
-	}
-
-	/* the project may load another list than the main setting */
-	if (session_files != NULL)
-	{
-		foreach_ptr_array(tmp_array, i, session_files)
-			g_strfreev(tmp_array);
-		g_ptr_array_free(session_files, TRUE);
-	}
-
-	session_files = g_ptr_array_new();
+	files = g_ptr_array_new();
 	have_session_files = TRUE;
 	i = 0;
 	while (have_session_files)
@@ -725,7 +816,7 @@ void configuration_load_session_files(GKeyFile *config, gboolean read_recent_fil
 			error = NULL;
 			have_session_files = FALSE;
 		}
-		g_ptr_array_add(session_files, tmp_array);
+		g_ptr_array_add(files, tmp_array);
 		i++;
 	}
 
@@ -738,6 +829,8 @@ void configuration_load_session_files(GKeyFile *config, gboolean read_recent_fil
 		g_free(tmp_string);
 	}
 #endif
+
+	return files;
 }
 
 
@@ -782,8 +875,7 @@ static void load_dialog_prefs(GKeyFile *config)
 	prefs.confirm_exit = utils_get_setting_boolean(config, PACKAGE, "pref_main_confirm_exit", FALSE);
 	prefs.suppress_status_messages = utils_get_setting_boolean(config, PACKAGE, "pref_main_suppress_status_messages", FALSE);
 	prefs.load_session = utils_get_setting_boolean(config, PACKAGE, "pref_main_load_session", TRUE);
-	project_prefs.project_session = utils_get_setting_boolean(config, PACKAGE, "pref_main_project_session", TRUE);
-	project_prefs.project_file_in_basedir = utils_get_setting_boolean(config, PACKAGE, "pref_main_project_file_in_basedir", FALSE);
+	project_prefs.project_file_in_basedir = utils_get_setting_boolean(config, PACKAGE, "pref_main_project_file_in_basedir", TRUE);
 	prefs.save_winpos = utils_get_setting_boolean(config, PACKAGE, "pref_main_save_winpos", TRUE);
 	prefs.save_wingeom = utils_get_setting_boolean(config, PACKAGE, "pref_main_save_wingeom", prefs.save_winpos);
 	prefs.beep_on_errors = utils_get_setting_boolean(config, PACKAGE, "beep_on_errors", TRUE);
@@ -803,7 +895,6 @@ static void load_dialog_prefs(GKeyFile *config)
 	interface_prefs.editor_font = utils_get_setting_string(config, PACKAGE, "editor_font", GEANY_DEFAULT_FONT_EDITOR);
 	interface_prefs.tagbar_font = utils_get_setting_string(config, PACKAGE, "tagbar_font", GEANY_DEFAULT_FONT_SYMBOL_LIST);
 	interface_prefs.msgwin_font = utils_get_setting_string(config, PACKAGE, "msgwin_font", GEANY_DEFAULT_FONT_MSG_WINDOW);
-	interface_prefs.use_native_windows_dialogs = utils_get_setting_boolean(config, PACKAGE, "use_native_windows_dialogs", FALSE);
 
 	/* display, editor */
 	editor_prefs.long_line_enabled = utils_get_setting_boolean(config, PACKAGE, "long_line_enabled", TRUE);
@@ -822,6 +913,7 @@ static void load_dialog_prefs(GKeyFile *config)
 	editor_prefs.show_indent_guide = utils_get_setting_boolean(config, PACKAGE, "show_indent_guide", FALSE);
 	editor_prefs.show_white_space = utils_get_setting_boolean(config, PACKAGE, "show_white_space", FALSE);
 	editor_prefs.show_line_endings = utils_get_setting_boolean(config, PACKAGE, "show_line_endings", FALSE);
+	editor_prefs.show_line_endings_only_when_differ = utils_get_setting_boolean(config, PACKAGE, "show_line_endings_only_when_differ", FALSE);
 	editor_prefs.scroll_stop_at_last_line = utils_get_setting_boolean(config, PACKAGE, "scroll_stop_at_last_line", TRUE);
 	editor_prefs.auto_close_xml_tags = utils_get_setting_boolean(config, PACKAGE, "auto_close_xml_tags", TRUE);
 	editor_prefs.complete_snippets = utils_get_setting_boolean(config, PACKAGE, "complete_snippets", TRUE);
@@ -891,7 +983,7 @@ static void load_dialog_prefs(GKeyFile *config)
 	vte_info.load_vte = utils_get_setting_boolean(config, "VTE", "load_vte", TRUE);
 	if (vte_info.load_vte && vte_info.load_vte_cmdline /* not disabled on the cmdline */)
 	{
-		StashGroup *group;
+		VteConfig *vc = &vte_config;
 		struct passwd *pw = getpwuid(getuid());
 		const gchar *shell = (pw != NULL) ? pw->pw_shell : "/bin/sh";
 
@@ -902,7 +994,6 @@ static void load_dialog_prefs(GKeyFile *config)
 			shell = "/bin/bash -l";
 #endif
 
-		vc = g_new0(VteConfig, 1);
 		vte_info.dir = utils_get_setting_string(config, "VTE", "last_dir", NULL);
 		if ((vte_info.dir == NULL || utils_str_equal(vte_info.dir, "")) && pw != NULL)
 			/* last dir is not set, fallback to user's home directory */
@@ -924,15 +1015,6 @@ static void load_dialog_prefs(GKeyFile *config)
 		vc->scrollback_lines = utils_get_setting_integer(config, "VTE", "scrollback_lines", 500);
 		get_setting_color(config, "VTE", "colour_fore", &vc->colour_fore, "#ffffff");
 		get_setting_color(config, "VTE", "colour_back", &vc->colour_back, "#000000");
-
-		/* various VTE prefs.
-		 * this can't be done in init_pref_groups() because we need to know the value of
-		 * vte_info.load_vte, and `vc` to be initialized */
-		group = stash_group_new("VTE");
-		configuration_add_various_pref_group(group, "terminal");
-
-		stash_group_add_string(group, &vc->send_cmd_prefix, "send_cmd_prefix", "");
-		stash_group_add_boolean(group, &vc->send_selection_unsafe, "send_selection_unsafe", FALSE);
 	}
 #endif
 	/* templates */
@@ -1004,27 +1086,15 @@ static void load_dialog_prefs(GKeyFile *config)
 	printing_prefs.print_page_header = utils_get_setting_boolean(config, "printing", "print_page_header", TRUE);
 	printing_prefs.page_header_basename = utils_get_setting_boolean(config, "printing", "page_header_basename", FALSE);
 	printing_prefs.page_header_datefmt = utils_get_setting_string(config, "printing", "page_header_datefmt", "%c");
-
-	/* read stash prefs */
-	settings_action(config, SETTING_READ);
-
-	/* build menu
-	 * after stash prefs as it uses some of them */
-	build_set_group_count(GEANY_GBG_FT, build_menu_prefs.number_ft_menu_items);
-	build_set_group_count(GEANY_GBG_NON_FT, build_menu_prefs.number_non_ft_menu_items);
-	build_set_group_count(GEANY_GBG_EXEC, build_menu_prefs.number_exec_menu_items);
-	build_load_menu(config, GEANY_BCS_PREF, NULL);
 }
 
 
 static void load_ui_prefs(GKeyFile *config)
 {
-	gint *geo;
-	gsize geo_len;
-
 	ui_prefs.sidebar_visible = utils_get_setting_boolean(config, PACKAGE, "sidebar_visible", TRUE);
 	ui_prefs.msgwindow_visible = utils_get_setting_boolean(config, PACKAGE, "msgwindow_visible", TRUE);
 	ui_prefs.fullscreen = utils_get_setting_boolean(config, PACKAGE, "fullscreen", FALSE);
+	ui_prefs.symbols_group_by_type = utils_get_setting_boolean(config, PACKAGE, "symbols_group_by_type", TRUE);
 	ui_prefs.custom_date_format = utils_get_setting_string(config, PACKAGE, "custom_date_format", "");
 	ui_prefs.custom_commands = g_key_file_get_string_list(config, PACKAGE, "custom_commands", NULL, NULL);
 	ui_prefs.custom_commands_labels = g_key_file_get_string_list(config, PACKAGE, "custom_commands_labels", NULL, NULL);
@@ -1068,6 +1138,12 @@ static void load_ui_prefs(GKeyFile *config)
 	scribble_text = utils_get_setting_string(config, PACKAGE, "scribble_text",
 				_("Type here what you want, use it as a notice/scratch board"));
 	scribble_pos = utils_get_setting_integer(config, PACKAGE, "scribble_pos", -1);
+}
+
+static void load_ui_session(GKeyFile *config)
+{
+	gint *geo;
+	gsize geo_len;
 
 	geo = g_key_file_get_integer_list(config, PACKAGE, "geometry", &geo_len, NULL);
 	if (! geo || geo_len < 5)
@@ -1103,7 +1179,7 @@ static void load_ui_prefs(GKeyFile *config)
  */
 void configuration_save_default_session(void)
 {
-	gchar *configfile = g_build_filename(app->configdir, "geany.conf", NULL);
+	gchar *configfile = g_build_filename(app->configdir, SESSION_FILE, NULL);
 	gchar *data;
 	GKeyFile *config = g_key_file_new();
 
@@ -1124,7 +1200,7 @@ void configuration_save_default_session(void)
 
 void configuration_clear_default_session(void)
 {
-	gchar *configfile = g_build_filename(app->configdir, "geany.conf", NULL);
+	gchar *configfile = g_build_filename(app->configdir, SESSION_FILE, NULL);
 	gchar *data;
 	GKeyFile *config = g_key_file_new();
 
@@ -1146,44 +1222,68 @@ void configuration_clear_default_session(void)
 /*
  * Only reload the session part of the default configuration
  */
-void configuration_reload_default_session(void)
+void configuration_load_default_session(void)
 {
-	gchar *configfile = g_build_filename(app->configdir, "geany.conf", NULL);
+	gchar *configfile = get_keyfile_for_payload(SESSION);
 	GKeyFile *config = g_key_file_new();
 
+	g_return_if_fail(default_session_files == NULL);
+
 	g_key_file_load_from_file(config, configfile, G_KEY_FILE_NONE, NULL);
 	g_free(configfile);
 
-	configuration_load_session_files(config, FALSE);
+	default_session_files = configuration_load_session_files(config);
 
 	g_key_file_free(config);
 }
 
+static gboolean read_config_file(ConfigPayload payload)
+{
+	GKeyFile *config = g_key_file_new();
+	gchar *configfile;
+
+	configfile = get_keyfile_for_payload(payload);
+	g_key_file_load_from_file(config, configfile, G_KEY_FILE_NONE, NULL);
+	g_free(configfile);
+
+	/* read stash prefs */
+	settings_action(config, SETTING_READ, payload);
+
+	switch (payload)
+	{
+		case PREFS:
+			load_dialog_prefs(config);
+			load_ui_prefs(config);
+
+			/* build menu, after stash prefs as it uses some of them */
+			build_set_group_count(GEANY_GBG_FT, build_menu_prefs.number_ft_menu_items);
+			build_set_group_count(GEANY_GBG_NON_FT, build_menu_prefs.number_non_ft_menu_items);
+			build_set_group_count(GEANY_GBG_EXEC, build_menu_prefs.number_exec_menu_items);
+			build_load_menu(config, GEANY_BCS_PREF, NULL);
+			/* this signal can be used e.g. to delay building UI elements until settings have been read */
+			g_signal_emit_by_name(geany_object, "load-settings", config);
+			break;
+		case SESSION:
+			project_load_prefs(config);
+			load_ui_session(config);
+			load_recent_files(config, ui_prefs.recent_queue, "recent_files");
+			load_recent_files(config, ui_prefs.recent_projects_queue, "recent_projects");
+			break;
+		case MAX_PAYLOAD:
+			g_assert_not_reached();
+	}
+
+	g_key_file_free(config);
+	return TRUE;
+}
+
 
 gboolean configuration_load(void)
 {
-	gchar *configfile = g_build_filename(app->configdir, "geany.conf", NULL);
-	GKeyFile *config = g_key_file_new();
+	gboolean prefs_loaded = read_config_file(PREFS);
+	gboolean sess_loaded = read_config_file(SESSION);
 
-	if (! g_file_test(configfile, G_FILE_TEST_IS_REGULAR))
-	{	/* config file does not (yet) exist, so try to load a global config file which may be */
-		/* created by distributors */
-		geany_debug("No user config file found, trying to use global configuration.");
-		SETPTR(configfile, g_build_filename(app->datadir, "geany.conf", NULL));
-	}
-	g_key_file_load_from_file(config, configfile, G_KEY_FILE_NONE, NULL);
-	g_free(configfile);
-
-	load_dialog_prefs(config);
-	load_ui_prefs(config);
-	project_load_prefs(config);
-	configuration_load_session_files(config, TRUE);
-
-	/* this signal can be used e.g. to delay building UI elements until settings have been read */
-	g_signal_emit_by_name(geany_object, "load-settings", config);
-
-	g_key_file_free(config);
-	return TRUE;
+	return prefs_loaded && sess_loaded;
 }
 
 
@@ -1254,16 +1354,14 @@ static gboolean open_session_file(gchar **tmp, guint len)
 /* Open session files
  * Note: notebook page switch handler and adding to recent files list is always disabled
  * for all files opened within this function */
-void configuration_open_files(void)
+void configuration_open_files(GPtrArray *session_files)
 {
-	gint i;
 	gboolean failure = FALSE;
 
 	/* necessary to set it to TRUE for project session support */
-	main_status.opening_session_files = TRUE;
+	main_status.opening_session_files++;
 
-	i = file_prefs.tab_order_ltr ? 0 : (session_files->len - 1);
-	while (TRUE)
+	for (guint i = 0; i < session_files->len; i++)
 	{
 		gchar **tmp = g_ptr_array_index(session_files, i);
 		guint len;
@@ -1274,42 +1372,29 @@ void configuration_open_files(void)
 				failure = TRUE;
 		}
 		g_strfreev(tmp);
-
-		if (file_prefs.tab_order_ltr)
-		{
-			i++;
-			if (i >= (gint)session_files->len)
-				break;
-		}
-		else
-		{
-			i--;
-			if (i < 0)
-				break;
-		}
 	}
 
 	g_ptr_array_free(session_files, TRUE);
-	session_files = NULL;
 
 	if (failure)
 		ui_set_statusbar(TRUE, _("Failed to load one or more session files."));
 	else
-	{
-		/* explicitly trigger a notebook page switch after unsetting main_status.opening_session_files
-		 * for callbacks to run (and update window title, encoding settings, and so on) */
-		gint n_pages = gtk_notebook_get_n_pages(GTK_NOTEBOOK(main_widgets.notebook));
-		gint cur_page = gtk_notebook_get_current_page(GTK_NOTEBOOK(main_widgets.notebook));
-		gint target_page = session_notebook_page >= 0 ? session_notebook_page : cur_page;
+		document_show_tab_idle(session_notebook_page >= 0 ? document_get_from_page(session_notebook_page) : document_get_current());
 
-		/* if target page is current page, switch to another page first to really trigger an event */
-		if (target_page == cur_page && n_pages > 0)
-			gtk_notebook_set_current_page(GTK_NOTEBOOK(main_widgets.notebook), (cur_page + 1) % n_pages);
+	session_notebook_page = -1;
+	main_status.opening_session_files--;
+}
 
-		main_status.opening_session_files = FALSE;
-		gtk_notebook_set_current_page(GTK_NOTEBOOK(main_widgets.notebook), target_page);
-	}
-	main_status.opening_session_files = FALSE;
+
+/* Open session files
+ * Note: notebook page switch handler and adding to recent files list is always disabled
+ * for all files opened within this function */
+void configuration_open_default_session(void)
+{
+	g_return_if_fail(default_session_files != NULL);
+
+	configuration_open_files(default_session_files);
+	default_session_files = NULL;
 }
 
 
@@ -1351,12 +1436,10 @@ void configuration_apply_settings(void)
 
 static gboolean save_configuration_cb(gpointer data)
 {
-	configuration_save();
 	if (app->project != NULL)
-	{
 		project_write_config();
-	}
-	document_list_update_idle_func_id = 0;
+	else
+		configuration_save_default_session();
 	return G_SOURCE_REMOVE;
 }
 
@@ -1372,17 +1455,16 @@ static void document_list_changed_cb(GObject *obj, GeanyDocument *doc, gpointer
 		!main_status.opening_session_files &&
 		!main_status.quitting)
 	{
-		if (document_list_update_idle_func_id == 0)
-		{
-			document_list_update_idle_func_id = g_idle_add(save_configuration_cb, NULL);
-		}
+		g_idle_remove_by_data(save_configuration_cb);
+		g_idle_add(save_configuration_cb, save_configuration_cb);
 	}
 }
 
 
 void configuration_init(void)
 {
-	keyfile_groups = g_ptr_array_new();
+	keyfile_groups[PREFS] = g_ptr_array_new_with_free_func((GDestroyNotify) stash_group_free);
+	keyfile_groups[SESSION] = g_ptr_array_new_with_free_func((GDestroyNotify) stash_group_free);
 	pref_groups = g_ptr_array_new();
 	init_pref_groups();
 
@@ -1394,14 +1476,9 @@ void configuration_init(void)
 
 void configuration_finalize(void)
 {
-	guint i;
-	StashGroup *group;
-
 	g_signal_handlers_disconnect_by_func(geany_object, G_CALLBACK(document_list_changed_cb), NULL);
 
-	foreach_ptr_array(group, i, keyfile_groups)
-		stash_group_free(group);
-
-	g_ptr_array_free(keyfile_groups, TRUE);
 	g_ptr_array_free(pref_groups, TRUE);
+	g_ptr_array_free(keyfile_groups[SESSION], TRUE);
+	g_ptr_array_free(keyfile_groups[PREFS], TRUE);
 }
