@@ -1,8 +1,7 @@
 /*
  *      stash.h - this file is part of Geany, a fast and lightweight IDE
  *
- *      Copyright 2008-2012 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
- *      Copyright 2008-2012 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
+ *      Copyright 2008 The Geany contributors
  *
  *      This program is free software; you can redistribute it and/or modify
  *      it under the terms of the GNU General Public License as published by
@@ -20,7 +19,9 @@
  */
 
 #ifndef GEANY_STASH_H
-#define GEANY_STASH_H
+#define GEANY_STASH_H 1
+
+#include "gtkcompat.h"
 
 G_BEGIN_DECLS
 
@@ -31,16 +32,16 @@ typedef struct StashGroup StashGroup;
  * stash_group_display() and stash_group_update(). */
 typedef gconstpointer StashWidgetID;
 
+GType stash_group_get_type(void);
 
 StashGroup *stash_group_new(const gchar *name);
 
-void stash_group_set_various(StashGroup *group, gboolean write_once);
-
-void stash_group_set_use_defaults(StashGroup *group, gboolean use_defaults);
-
 void stash_group_add_boolean(StashGroup *group, gboolean *setting,
 		const gchar *key_name, gboolean default_value);
 
+void stash_group_add_double(StashGroup *group, gdouble *setting,
+		const gchar *key_name, gdouble default_value);
+
 void stash_group_add_integer(StashGroup *group, gint *setting,
 		const gchar *key_name, gint default_value);
 
@@ -54,16 +55,13 @@ void stash_group_load_from_key_file(StashGroup *group, GKeyFile *keyfile);
 
 void stash_group_save_to_key_file(StashGroup *group, GKeyFile *keyfile);
 
+void stash_group_free(StashGroup *group);
+
 gboolean stash_group_load_from_file(StashGroup *group, const gchar *filename);
 
 gint stash_group_save_to_file(StashGroup *group, const gchar *filename,
 		GKeyFileFlags flags);
 
-void stash_group_free_settings(StashGroup *group);
-
-void stash_group_free(StashGroup *group);
-
-
 /* *** GTK-related functions *** */
 
 void stash_group_add_toggle_button(StashGroup *group, gboolean *setting,
@@ -93,12 +91,26 @@ void stash_group_display(StashGroup *group, GtkWidget *owner);
 
 void stash_group_update(StashGroup *group, GtkWidget *owner);
 
+void stash_group_free_settings(StashGroup *group);
+
+
+#ifdef GEANY_PRIVATE
+
+void stash_group_set_various(StashGroup *group, gboolean various,
+	const gchar *prefix);
+
+void stash_group_set_use_defaults(StashGroup *group, gboolean use_defaults);
+
+/* *** GTK-related functions *** */
+
 void stash_tree_setup(GPtrArray *group_array, GtkTreeView *tree);
 
 void stash_tree_display(GtkTreeView *tree);
 
 void stash_tree_update(GtkTreeView *tree);
 
+#endif /* GEANY_PRIVATE */
+
 G_END_DECLS
 
-#endif
+#endif /* GEANY_STASH_H */
